{**********************************************************************
                Copyright (c) PilotLogic Software House

 Package pl_FFmpeg5
 This file is part of CodeTyphon Studio (https://www.pilotlogic.com/)

 ***** BEGIN LICENSE BLOCK *****
 * Version: LGPL-2.1 with link exception (Modified LGPL)
 *
 * The contents of this file are subject to the
 * GNU LIBRARY GENERAL PUBLIC LICENSE Version 2.1 (the "License")
 * with the following modification:
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent modules,
 * and to copy and distribute the resulting executable under terms of your choice,
 * provided that you also meet, for each linked independent module, the terms
 * and conditions of the license of that module. An independent module is a
 * module which is not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library, but
 * you are not obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * ***** END LICENSE BLOCK *****

***********************************************************************}

unit ffmpeg_loader;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,
  sysloader,

  libavdevice,
  libavfilter,
  libpostproc,
  libswscale,
  libswresample,
  libavformat,
  libavcodec,
  libavutil;

Type

  TLoaderFFmpeg = Class(TBaseLoader)
  private
  public     
    constructor Create(AOwner: TComponent); override;

    Procedure Libraries_Load; override;
    Procedure Libraries_Free; override;

    Function  GetlibavDevice_Loaded:Boolean;
    Function  GetlibavFilter_Loaded:Boolean;
    Function  GetlibPostproc_Loaded:Boolean;
    Function  GetlibSwScale_Loaded:Boolean;
    Function  GetlibSwResample_Loaded:Boolean;
    Function  GetlibavFormat_Loaded:Boolean;
    Function  GetlibavCodec_Loaded:Boolean;
    Function  GetlibavUtil_Loaded:Boolean;
  published
    Property IslibavDevice_Loaded:Boolean read GetlibavDevice_Loaded stored False;
    Property IslibavFilter_Loaded:Boolean read GetlibavFilter_Loaded stored False;
    Property IslibPostproc_Loaded:Boolean read GetlibPostproc_Loaded stored False;
    Property IslibSwScale_Loaded:Boolean read GetlibSwScale_Loaded stored False;
    Property IslibSwResample_Loaded:Boolean read GetlibSwResample_Loaded stored False;
    Property IslibavFormat_Loaded:Boolean read GetlibavFormat_Loaded stored False;
    Property IslibavCodec_Loaded:Boolean read GetlibavCodec_Loaded stored False;
    Property IslibavUtil_Loaded:Boolean read GetlibavUtil_Loaded stored False;
  end;

Function  IsLibFFmpeg_Loaded: boolean;

implementation

Function  IsLibFFmpeg_Loaded: boolean;
begin
  Result:=libavDevice_IsLoaded and
          libavFilter_IsLoaded and
          libPostproc_IsLoaded and
          libSwScale_IsLoaded and
          libSwResample_IsLoaded and
          libavFormat_IsLoaded and
          libavCodec_IsLoaded and
          libavUtil_IsLoaded;
end;

//========== TLoaderFFmpeg ==========================

constructor TLoaderFFmpeg.Create(AOwner: TComponent);
begin
  inherited Create(AOwner); 
  FCTSubDir:='ffmpeg5';
end;

Procedure TLoaderFFmpeg.Libraries_Load;
begin
  inherited Libraries_Load;
  //.............................

  libavdevice_Load;
  libavfilter_Load;
  libpostproc_Load;
  libswscale_Load;
  libswresample_Load; 
  libavformat_Load;
  libavcodec_Load;
  libavutil_Load;

  FActive:=True;
end;

Procedure TLoaderFFmpeg.Libraries_Free;
begin

  libavdevice_Free;
  libavfilter_Free;
  libpostproc_Free;
  libswscale_Free;
  libswresample_Free;
  libavformat_Free;
  libavcodec_Free;
  libavutil_Free;

  FActive:=False;
                   
  //.............................
  inherited Libraries_Free;
end;

Function TLoaderFFmpeg.GetlibavDevice_Loaded:Boolean;
begin
  Result:=libavDevice_IsLoaded;
end;

Function TLoaderFFmpeg.GetlibavFilter_Loaded:Boolean;
begin
  Result:=libavFilter_IsLoaded;
end;

Function TLoaderFFmpeg.GetlibPostproc_Loaded:Boolean;
begin
  Result:=libPostproc_IsLoaded;
end;

Function TLoaderFFmpeg.GetlibSwScale_Loaded:Boolean;
begin
  Result:=libSwScale_IsLoaded;
end;

Function TLoaderFFmpeg.GetlibSwResample_Loaded:Boolean;
begin
  Result:=libSwResample_IsLoaded;
end;

Function TLoaderFFmpeg.GetlibavFormat_Loaded:Boolean;
begin
  Result:=libavFormat_IsLoaded;
end;

Function TLoaderFFmpeg.GetlibavCodec_Loaded:Boolean;
begin
  Result:=libavCodec_IsLoaded;
end;

Function TLoaderFFmpeg.GetlibavUtil_Loaded:Boolean;
begin
  Result:=libavUtil_IsLoaded;
end;



end.

