{**********************************************************************
                Copyright (c) PilotLogic Software House

 Package pl_FFmpeg5
 This file is part of CodeTyphon Studio (https://www.pilotlogic.com/)

 ***** BEGIN LICENSE BLOCK *****
 * Version: LGPL-2.1 with link exception (Modified LGPL)
 *
 * The contents of this file are subject to the
 * GNU LIBRARY GENERAL PUBLIC LICENSE Version 2.1 (the "License")
 * with the following modification:
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent modules,
 * and to copy and distribute the resulting executable under terms of your choice,
 * provided that you also meet, for each linked independent module, the terms
 * and conditions of the license of that module. An independent module is a
 * module which is not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library, but
 * you are not obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * ***** END LICENSE BLOCK *****

***********************************************************************}

Unit libavfilter;

{$MODE objfpc}{$H+}

Interface

Uses
  ctypes,
  Classes, dynlibs,
  ffmpeg_types, libavutil, libavformat;

Const
{$IF defined(MSWINDOWS)}
   avfilterLibName = 'avfilter-8.dll';
{$ELSEIF defined(LINUX)}
   avfilterLibName = 'libavfilter.so.8';
{$ELSEIF defined(DARWIN)}
   avfilterLibName = 'libavfilter.8.dylib';
{$ELSE}
   avfilterLibName = 'libavfilter.so.8';
{$ENDIF}


Const

   AVFILTER_THREAD_SLICE = (1 Shl 0);
   AVFILTER_FLAG_DYNAMIC_INPUTS = (1 Shl 0);
   AVFILTER_FLAG_DYNAMIC_OUTPUTS = (1 Shl 1);
   AVFILTER_FLAG_SLICE_THREADS = (1 Shl 2);
   AVFILTER_FLAG_SUPPORT_TIMELINE_GENERIC = (1 Shl 16);
   AVFILTER_FLAG_SUPPORT_TIMELINE_INTERNAL = (1 Shl 17);
   AVFILTER_FLAG_SUPPORT_TIMELINE = (AVFILTER_FLAG_SUPPORT_TIMELINE_GENERIC Or AVFILTER_FLAG_SUPPORT_TIMELINE_INTERNAL);
   AVFILTER_AUTO_CONVERT_ALL = 0;
   AVFILTER_AUTO_CONVERT_NONE = -1;
   AV_BUFFERSINK_FLAG_PEEK = 1;
   AV_BUFFERSINK_FLAG_NO_REQUEST = 2;
   AV_BUFFERSRC_FLAG_NO_CHECK_FORMAT = 1;
   AV_BUFFERSRC_FLAG_PUSH = 4;
   AV_BUFFERSRC_FLAG_KEEP_REF = 8;

Type
   pAVFilterContext = ^AVFilterContext;
   ppAVFilterContext = ^pAVFilterContext;
   pAVFilterLink = ^AVFilterLink;
   ppAVFilterLink = ^pAVFilterLink;

   pAVFilterPad = ^AVFilterPad;
   AVFilterPad = Record
   End;
   
   ppAVFilterGraph = ^pAVFilterGraph;
   pAVFilterGraph = ^AVFilterGraph;

   pAVFilterCommand = ^AVFilterCommand;
   AVFilterCommand = Record
   End;

   pAVFilter = ^AVFilter;
   AVFilter = Record
      name: pAnsiChar;
      description: pAnsiChar;
      inputs: pAVFilterPad;
      outputs: pAVFilterPad;
      priv_class: pAVClass;
      flags: int;
      preinit: Function(ctx: pAVFilterContext): int; cdecl;
      init: Function(ctx: pAVFilterContext): int; cdecl;
      init_dict: Function(ctx: pAVFilterContext; Var options: pAVDictionary): int; cdecl;
      uninit: Procedure(ctx: pAVFilterContext); cdecl;
      query_formats: Function(p: pAVFilterContext): int; cdecl;
      priv_size: int;
      flags_internal: int;
      next: pAVFilter;
      process_command: Function(ctx: pAVFilterContext; Const cmd: pAnsiChar; Const arg: pAnsiChar; res: pAnsiChar; res_len: int; flags: int): int; cdecl;
      init_opaque: Function(ctx: pAVFilterContext; opaque: pointer): int; cdecl;
      activate: Function(ctx: pAVFilterContext): int; cdecl;
   End;

   pAVFilterInternal = ^AVFilterInternal;
   AVFilterInternal = Record
   End;

   AVFilterContext = Record
      av_class: pAVClass;
      filter: pAVFilter;
      name: pAnsiChar;
      input_pads: pAVFilterPad;
      inputs: ppAVFilterLink;
      nb_inputs: unsigned;
      output_pads: pAVFilterPad;
      outputs: ppAVFilterLink;
      nb_outputs: unsigned;
      priv: pointer;
      graph: pAVFilterGraph;
      thread_type: int;
      internal: pAVFilterInternal;
      command_queue: pAVFilterCommand;
      enable_str: pAnsiChar;
      enable: pointer;
      var_values: pdouble;
      is_disabled: int;
      hw_device_ctx: pAVBufferRef;
      nb_threads: int;
      ready: unsigned;
      extra_hw_frames: int;
   End;

   pAVFilterGraphInternal = ^AVFilterGraphInternal;
   AVFilterGraphInternal = Record
   End;

   Tavfilter_action_func = Function(ctx: pAVFilterContext; arg: pointer; jobnr: int; b_jobs: int): int; cdecl;
   Tavfilter_execute_func = Function(ctx: pAVFilterContext; func: Tavfilter_action_func; arg: pointer; Var ret: int; nb_jobs: int): int; cdecl;

   AVFilterGraph = Record
      av_class: pAVClass;
      filters: ppAVFilterContext;
      nb_filters: unsigned;
      scale_sws_opts: pAnsiChar;
      thread_type: int;
      nb_threads: int;
      internal: pAVFilterGraphInternal;
      opaque: pointer;
      execute: Tavfilter_execute_func;
      aresample_swr_opts: pAnsiChar;
      sink_links: ppAVFilterLink;
      sink_links_count: int;
      disable_auto_convert: unsigned;
   End;

   Tinit_state = (
      AVLINK_UNINIT = 0,
      AVLINK_STARTINIT,
      AVLINK_INIT
      );

   AVFilterLink = Record
      src: pAVFilterContext;
      srcpad: pAVFilterPad;
      dst: pAVFilterContext;
      dstpad: pAVFilterPad;
      _type: AVMediaType;
      w: int;
      h: int;
      sample_aspect_ratio: AVRational;
      channel_layout: uint64_t;
      sample_rate: int;
      format: int;
      time_base: AVRational;
      in_formats: pAVFilterFormats;
      out_formats: pAVFilterFormats;
      in_samplerates: pAVFilterFormats;
      out_samplerates: pAVFilterFormats;
      in_channel_layouts: pAVFilterChannelLayouts;
      out_channel_layouts: pAVFilterChannelLayouts;
      request_samples: int;
      init_state: Tinit_state;
      graph: pAVFilterGraph;
      current_pts: int64_t;
      current_pts_us: int64_t;
      age_index: int;
      frame_rate: AVRational;
      partial_buf: pAVFrame;
      partial_buf_size: int;
      min_samples: int;
      max_samples: int;
      channels: int;
      flags: unsigned;
      frame_count_in, frame_count_out: int64_t;
      frame_pool: pointer;
      frame_wanted_out: int;
      hw_frames_ctx: pAVBufferRef;
      fifo: FFFrameQueue;
      frame_blocked_in: int;
      status_in: int;
      status_in_pts: int64_t;
      status_out: int;
   End;

   pAVFilterInOut = ^AVFilterInOut;
   AVFilterInOut = Record
      name: pAnsiChar;
      filter_ctx: pAVFilterContext;
      pad_idx: int;
      next: pAVFilterInOut;
   End;

   pAVBufferSinkParams = ^AVBufferSinkParams;
   AVBufferSinkParams = Record
      pixel_fmts: pAVPixelFormat;
   End;


   pAVABufferSinkParams = ^AVABufferSinkParams;
   AVABufferSinkParams = Record
      sample_fmts: pAVSampleFormat;
      channel_layouts: pint64_t;
      channel_counts: pint;
      all_channel_counts: int;
      sample_rates: pint;
   End;

   pAVBufferSrcParameters = ^AVBufferSrcParameters;
   AVBufferSrcParameters = Record
      format: int;
      time_base: AVRational;
      width, height: int;
      sample_aspect_ratio: AVRational;
      frame_rate: AVRational;
      hw_frames_ctx: pAVBufferRef;
      sample_rate: int;
      channel_layout: uint64_t;
   End;

//-------------------------------------------------------------------------
Type

   Tavfilter_version = Function(): unsigned; cdecl;
   Tavfilter_configuration = Function(): pAnsiChar; cdecl;
   Tavfilter_license = Function(): pAnsiChar; cdecl;
   Tavfilter_link = Function(src: pAVFilterContext; srcpad: unsigned; dst: pAVFilterContext; dstpad: unsigned): int; cdecl;
   Tavfilter_link_free = Procedure(Var link: pAVFilterLink); cdecl;
   Tavfilter_config_links = Function(filter: pAVFilterContext): int; cdecl;
   Tavfilter_process_command = Function(filter: pAVFilterContext; Const cmd: pAnsiChar; Const arg: pAnsiChar;
                                        res: pAnsiChar; res_len: int; flags: int): int; cdecl;
   Tav_filter_iterate = Function(Var opaque: pointer): pAVFilter; cdecl;
   Tavfilter_get_by_name = Function(Const name: pAnsiChar): pAVFilter; cdecl;
   Tavfilter_init_str = Function(ctx: pAVFilterContext; Const args: pAnsiChar): int; cdecl;
   Tavfilter_init_dict = Function(ctx: pAVFilterContext; Var options: pAVDictionary): int; cdecl;
   Tavfilter_free = Procedure(filter: pAVFilterContext); cdecl;
   Tavfilter_insert_filter = Function(link: pAVFilterLink; filt: pAVFilterContext; filt_srcpad_idx: unsigned; filt_dstpad_idx: unsigned): int; cdecl;
   Tavfilter_get_class = Function(): pAVClass; cdecl;
   Tavfilter_graph_alloc = Function(): pAVFilterGraph; cdecl;
   Tavfilter_graph_alloc_filter = Function(graph: pAVFilterGraph; Const filter: pAVFilter; Const name: pAnsiChar): pAVFilterContext; cdecl;
   Tavfilter_graph_get_filter = Function(graph: pAVFilterGraph; Const name: pAnsiChar): pAVFilterContext; cdecl;
   Tavfilter_graph_create_filter = Function(Var filt_ctx: pAVFilterContext; Const filt: pAVFilter; Const name: pAnsiChar;
                                           Const args: pAnsiChar; opaque: pointer; graph_ctx: pAVFilterGraph): int; cdecl;
   Tavfilter_graph_set_auto_convert = Procedure(graph: pAVFilterGraph; flags: unsigned); cdecl;
   Tavfilter_graph_config = Function(graphctx: pAVFilterGraph; log_ctx: pointer): int; cdecl;
   Tavfilter_graph_free = Procedure(Var graph: pAVFilterGraph); cdecl;
   Tavfilter_inout_alloc = Function(): pAVFilterInOut; cdecl;
   Tavfilter_inout_free = Procedure(Var inout: pAVFilterInOut); cdecl;
   Tavfilter_graph_parse = Function(graph: pAVFilterGraph; Const filters: pAnsiChar; inputs: pAVFilterInOut; outputs: pAVFilterInOut; log_ctx: pointer): int; cdecl;
   Tavfilter_graph_parse_ptr = Function(graph: pAVFilterGraph; Const filters: pAnsiChar; Var inputs: pAVFilterInOut;
                                           Var outputs: pAVFilterInOut; log_ctx: pointer): int; cdecl;
   Tavfilter_graph_parse2 = Function(graph: pAVFilterGraph; Const filters: pAnsiChar; Var inputs: pAVFilterInOut; Var outputs: pAVFilterInOut): int; cdecl;
   Tavfilter_graph_send_command = Function(graph: pAVFilterGraph; Const target: pAnsiChar; Const cmd: pAnsiChar; Const arg: pAnsiChar;
                                           res: pAnsiChar; res_len: int; flags: int): int; cdecl;
   Tavfilter_graph_queue_command = Function(graph: pAVFilterGraph; Const target: pAnsiChar; Const cmd: pAnsiChar; Const arg: pAnsiChar;
                                            flags: int; ts: double): int; cdecl;
   Tavfilter_graph_dump = Function(graph: pAVFilterGraph; Const options: pAnsiChar): pAnsiChar; cdecl;
   Tavfilter_graph_request_oldest = Function(graph: pAVFilterGraph): int; cdecl;
   Tav_buffersink_get_frame_flags = Function(ctx: pAVFilterContext; frame: pAVFrame; flags: int): int; cdecl;
   Tav_buffersink_set_frame_size = Procedure(ctx: pAVFilterContext; frame_size: unsigned); cdecl;
   Tav_buffersink_get_type = Function(Const ctx: pAVFilterContext): AVMediaType; cdecl;
   Tav_buffersink_get_time_base = Function(Const ctx: pAVFilterContext): AVRational; cdecl;
   Tav_buffersink_get_format = Function(Const ctx: pAVFilterContext): int; cdecl;
   Tav_buffersink_get_frame_rate = Function(Const ctx: pAVFilterContext): AVRational; cdecl;
   Tav_buffersink_get_w = Function(Const ctx: pAVFilterContext): int; cdecl;
   Tav_buffersink_get_h = Function(Const ctx: pAVFilterContext): int; cdecl;
   Tav_buffersink_get_sample_aspect_ratio = Function(Const ctx: pAVFilterContext): AVRational; cdecl;
   Tav_buffersink_get_channels = Function(Const ctx: pAVFilterContext): int; cdecl;
   Tav_buffersink_get_channel_layout = Function(Const ctx: pAVFilterContext): uint64_t; cdecl;
   Tav_buffersink_get_sample_rate = Function(Const ctx: pAVFilterContext): int; cdecl;
   Tav_buffersink_get_hw_frames_ctx = Function(Const ctx: pAVFilterContext): pAVBufferRef; cdecl;
   Tav_buffersink_get_frame = Function(ctx: pAVFilterContext; frame: pAVFrame): int; cdecl;
   Tav_buffersink_get_samples = Function(ctx: pAVFilterContext; frame: pAVFrame; nb_samples: int): int; cdecl;
   Tav_buffersrc_get_nb_failed_requests = Function(buffer_src: pAVFilterContext): unsigned; cdecl;
   Tav_buffersrc_parameters_alloc = Function(): pAVBufferSrcParameters; cdecl;
   Tav_buffersrc_parameters_set = Function(ctx: pAVFilterContext; param: pAVBufferSrcParameters): int; cdecl;
   Tav_buffersrc_write_frame = Function(ctx: pAVFilterContext; Const frame: pAVFrame): int; cdecl;
   Tav_buffersrc_add_frame = Function(ctx: pAVFilterContext; frame: pAVFrame): int; cdecl;
   Tav_buffersrc_add_frame_flags = Function(buffer_src: pAVFilterContext; frame: pAVFrame; flags: int): int; cdecl;
   Tav_buffersrc_close = Function(ctx: pAVFilterContext; pts: int64_t; flags: unsigned): int; cdecl;      
   Tavfilter_filter_pad_count = Function(Const filter: PAVFilter; is_output: Integer): Cardinal; cdecl;
   Tavfilter_pad_get_name = Function(Const pads: PAVFilterPad; pad_idx: Integer): PAnsiChar; cdecl;
          
//-------------------------------------------------
 Var

   avfilter_version : Tavfilter_version = Nil;
   avfilter_configuration : Tavfilter_configuration = Nil;
   avfilter_license : Tavfilter_license = Nil;
   avfilter_link : Tavfilter_link = Nil;
   avfilter_link_free : Tavfilter_link_free = Nil;
   avfilter_config_links : Tavfilter_config_links = Nil;
   avfilter_process_command : Tavfilter_process_command = Nil;
   av_filter_iterate : Tav_filter_iterate = Nil;
   avfilter_get_by_name : Tavfilter_get_by_name = Nil;
   avfilter_init_str : Tavfilter_init_str = Nil;
   avfilter_init_dict : Tavfilter_init_dict = Nil;
   avfilter_free : Tavfilter_free = Nil;
   avfilter_insert_filter : Tavfilter_insert_filter = Nil;
   avfilter_get_class : Tavfilter_get_class = Nil;
   avfilter_graph_alloc : Tavfilter_graph_alloc = Nil;
   avfilter_graph_alloc_filter : Tavfilter_graph_alloc_filter = Nil;
   avfilter_graph_get_filter : Tavfilter_graph_get_filter = Nil;
   avfilter_graph_create_filter : Tavfilter_graph_create_filter = Nil;
   avfilter_graph_set_auto_convert : Tavfilter_graph_set_auto_convert = Nil;
   avfilter_graph_config : Tavfilter_graph_config = Nil;
   avfilter_graph_free : Tavfilter_graph_free = Nil;
   avfilter_inout_alloc : Tavfilter_inout_alloc = Nil;
   avfilter_inout_free : Tavfilter_inout_free = Nil;
   avfilter_graph_parse : Tavfilter_graph_parse = Nil;
   avfilter_graph_parse_ptr : Tavfilter_graph_parse_ptr = Nil;
   avfilter_graph_parse2 : Tavfilter_graph_parse2 = Nil;
   avfilter_graph_send_command : Tavfilter_graph_send_command = Nil;
   avfilter_graph_queue_command : Tavfilter_graph_queue_command = Nil;
   avfilter_graph_dump : Tavfilter_graph_dump = Nil;
   avfilter_graph_request_oldest : Tavfilter_graph_request_oldest = Nil;
   av_buffersink_get_frame_flags : Tav_buffersink_get_frame_flags = Nil;
   av_buffersink_set_frame_size : Tav_buffersink_set_frame_size = Nil;
   av_buffersink_get_type : Tav_buffersink_get_type = Nil;
   av_buffersink_get_time_base : Tav_buffersink_get_time_base = Nil;
   av_buffersink_get_format : Tav_buffersink_get_format = Nil;
   av_buffersink_get_frame_rate : Tav_buffersink_get_frame_rate = Nil;
   av_buffersink_get_w : Tav_buffersink_get_w = Nil;
   av_buffersink_get_h : Tav_buffersink_get_h = Nil;
   av_buffersink_get_sample_aspect_ratio : Tav_buffersink_get_sample_aspect_ratio = Nil;
   av_buffersink_get_channels : Tav_buffersink_get_channels = Nil;
   av_buffersink_get_channel_layout : Tav_buffersink_get_channel_layout = Nil;
   av_buffersink_get_sample_rate : Tav_buffersink_get_sample_rate = Nil;
   av_buffersink_get_hw_frames_ctx : Tav_buffersink_get_hw_frames_ctx = Nil;
   av_buffersink_get_frame : Tav_buffersink_get_frame = Nil;
   av_buffersink_get_samples : Tav_buffersink_get_samples = Nil;
   av_buffersrc_get_nb_failed_requests : Tav_buffersrc_get_nb_failed_requests = Nil;
   av_buffersrc_parameters_alloc : Tav_buffersrc_parameters_alloc = Nil;
   av_buffersrc_parameters_set : Tav_buffersrc_parameters_set = Nil;
   av_buffersrc_write_frame : Tav_buffersrc_write_frame = Nil;
   av_buffersrc_add_frame : Tav_buffersrc_add_frame = Nil;
   av_buffersrc_add_frame_flags : Tav_buffersrc_add_frame_flags = Nil;
   av_buffersrc_close : Tav_buffersrc_close = Nil;   
   avfilter_pad_get_name : Tavfilter_pad_get_name = Nil;
   avfilter_filter_pad_count : Tavfilter_filter_pad_count = Nil;

//-------------------------------------------------

Function  libavfilter_Load(Const aLibName: String = avfilterLibName): Boolean;
Procedure libavfilter_Free;
Function  libavfilter_IsLoaded: boolean;

//==============================================================
//==============================================================
implementation

Const
   InvalidLibHandle = 0;

var
  avfilter_Handle: TLibHandle;

function libavfilter_IsLoaded: boolean;
begin
  Result := (avfilter_Handle <> InvalidLibHandle);
end;

Procedure libavfilter_Free;
begin
  if libavfilter_IsLoaded then
  begin
    DynLibs.UnloadLibrary(avfilter_Handle);
    avfilter_Handle := InvalidLibHandle;
  end;
end;

function libavfilter_Load(Const aLibName: String = avfilterLibName): Boolean;
//.............................
   Function _GetProcAddress(Const aProcName: String): {$IFDEF cpui8086}FarPointer{$ELSE}Pointer{$ENDIF};
   Begin
      result := Nil;
      If aProcName = '' Then
         exit;

      result := dynlibs.GetProcedureAddress(avfilter_Handle, AnsiString(aProcName));
   End;
   //.....................................
var
  thelib: string;

{$IF DEFINED(UNIX) and (NOT DEFINED(DARWIN))}
  ssl:Integer;
  ssp:Integer;
{$ENDIF}
begin
  Result := False;

  if avfilter_Handle <> InvalidLibHandle then Exit(True);

  if aLibName = '' then
     thelib := avfilterLibName else
     thelib := aLibName;

  avfilter_Handle := dynlibs.LoadLibrary(thelib);

   //.....................................
   // On UNIX If Library NOT load, try to load .so file
  {$IF DEFINED(UNIX) and (NOT DEFINED(DARWIN))}
   If avfilter_Handle = 0 Then
   Begin
      ssl := length(thelib);
      ssp := Pos('.so.', thelib);
      If ssp > 0 Then
         delete(thelib, ssp, ssl - ssp + 1);

      avfilter_Handle := dynlibs.LoadLibrary(thelib + '.so');
   End;
  {$ENDIF}

    if avfilter_Handle = InvalidLibHandle then Exit(False);

    //.........................................

    Pointer(avfilter_version) := _GetProcAddress('avfilter_version');
    Pointer(avfilter_configuration) := _GetProcAddress('avfilter_configuration');
    Pointer(avfilter_license) := _GetProcAddress('avfilter_license');
    Pointer(avfilter_link) := _GetProcAddress('avfilter_link');
    Pointer(avfilter_link_free) := _GetProcAddress('avfilter_link_free');
    Pointer(avfilter_config_links) := _GetProcAddress('avfilter_config_links');
    Pointer(avfilter_process_command) := _GetProcAddress('avfilter_process_command');
    Pointer(av_filter_iterate) := _GetProcAddress('av_filter_iterate');
    Pointer(avfilter_get_by_name) := _GetProcAddress('avfilter_get_by_name');
    Pointer(avfilter_init_str) := _GetProcAddress('avfilter_init_str');
    Pointer(avfilter_init_dict) := _GetProcAddress('avfilter_init_dict');
    Pointer(avfilter_free) := _GetProcAddress('avfilter_free');
    Pointer(avfilter_insert_filter) := _GetProcAddress('avfilter_insert_filter');
    Pointer(avfilter_get_class) := _GetProcAddress('avfilter_get_class');
    Pointer(avfilter_graph_alloc) := _GetProcAddress('avfilter_graph_alloc');
    Pointer(avfilter_graph_alloc_filter) := _GetProcAddress('avfilter_graph_alloc_filter');
    Pointer(avfilter_graph_get_filter) := _GetProcAddress('avfilter_graph_get_filter');
    Pointer(avfilter_graph_create_filter) := _GetProcAddress('avfilter_graph_create_filter');
    Pointer(avfilter_graph_set_auto_convert) := _GetProcAddress('avfilter_graph_set_auto_convert');
    Pointer(avfilter_graph_config) := _GetProcAddress('avfilter_graph_config');
    Pointer(avfilter_graph_free) := _GetProcAddress('avfilter_graph_free');
    Pointer(avfilter_inout_alloc) := _GetProcAddress('avfilter_inout_alloc');
    Pointer(avfilter_inout_free) := _GetProcAddress('avfilter_inout_free');
    Pointer(avfilter_graph_parse) := _GetProcAddress('avfilter_graph_parse');
    Pointer(avfilter_graph_parse_ptr) := _GetProcAddress('avfilter_graph_parse_ptr');
    Pointer(avfilter_graph_parse2) := _GetProcAddress('avfilter_graph_parse2');
    Pointer(avfilter_graph_send_command) := _GetProcAddress('avfilter_graph_send_command');
    Pointer(avfilter_graph_queue_command) := _GetProcAddress('avfilter_graph_queue_command');
    Pointer(avfilter_graph_dump) := _GetProcAddress('avfilter_graph_dump');
    Pointer(avfilter_graph_request_oldest) := _GetProcAddress('avfilter_graph_request_oldest');
    Pointer(av_buffersink_get_frame_flags) := _GetProcAddress('av_buffersink_get_frame_flags');
    Pointer(av_buffersink_set_frame_size) := _GetProcAddress('av_buffersink_set_frame_size');
    Pointer(av_buffersink_get_type) := _GetProcAddress('av_buffersink_get_type');
    Pointer(av_buffersink_get_time_base) := _GetProcAddress('av_buffersink_get_time_base');
    Pointer(av_buffersink_get_format) := _GetProcAddress('av_buffersink_get_format');
    Pointer(av_buffersink_get_frame_rate) := _GetProcAddress('av_buffersink_get_frame_rate');
    Pointer(av_buffersink_get_w) := _GetProcAddress('av_buffersink_get_w');
    Pointer(av_buffersink_get_h) := _GetProcAddress('av_buffersink_get_h');
    Pointer(av_buffersink_get_sample_aspect_ratio) := _GetProcAddress('av_buffersink_get_sample_aspect_ratio');
    Pointer(av_buffersink_get_channels) := _GetProcAddress('av_buffersink_get_channels');
    Pointer(av_buffersink_get_channel_layout) := _GetProcAddress('av_buffersink_get_channel_layout');
    Pointer(av_buffersink_get_sample_rate) := _GetProcAddress('av_buffersink_get_sample_rate');
    Pointer(av_buffersink_get_hw_frames_ctx) := _GetProcAddress('av_buffersink_get_hw_frames_ctx');
    Pointer(av_buffersink_get_frame) := _GetProcAddress('av_buffersink_get_frame');
    Pointer(av_buffersink_get_samples) := _GetProcAddress('av_buffersink_get_samples');
    Pointer(av_buffersrc_get_nb_failed_requests) := _GetProcAddress('av_buffersrc_get_nb_failed_requests');
    Pointer(av_buffersrc_parameters_alloc) := _GetProcAddress('av_buffersrc_parameters_alloc');
    Pointer(av_buffersrc_parameters_set) := _GetProcAddress('av_buffersrc_parameters_set');
    Pointer(av_buffersrc_write_frame) := _GetProcAddress('av_buffersrc_write_frame');
    Pointer(av_buffersrc_add_frame) := _GetProcAddress('av_buffersrc_add_frame');
    Pointer(av_buffersrc_add_frame_flags) := _GetProcAddress('av_buffersrc_add_frame_flags');
    Pointer(av_buffersrc_close) := _GetProcAddress('av_buffersrc_close'); 
    Pointer(avfilter_pad_get_name) := _GetProcAddress('avfilter_pad_get_name');
    Pointer(avfilter_filter_pad_count) := _GetProcAddress('avfilter_filter_pad_count');


    //.........................................

    Result := True;

end;

end.
