{**********************************************************************
                Copyright (c) PilotLogic Software House

 Package pl_FFmpeg5
 This file is part of CodeTyphon Studio (https://www.pilotlogic.com/)

 ***** BEGIN LICENSE BLOCK *****
 * Version: LGPL-2.1 with link exception (Modified LGPL)
 *
 * The contents of this file are subject to the
 * GNU LIBRARY GENERAL PUBLIC LICENSE Version 2.1 (the "License")
 * with the following modification:
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent modules,
 * and to copy and distribute the resulting executable under terms of your choice,
 * provided that you also meet, for each linked independent module, the terms
 * and conditions of the license of that module. An independent module is a
 * module which is not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library, but
 * you are not obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * ***** END LICENSE BLOCK *****

***********************************************************************}

Unit libavformat;

{$MODE objfpc}{$H+}

Interface

Uses
  ctypes,
  Classes, dynlibs,
  ffmpeg_types, libavutil, libavcodec;

Const
{$IF defined(MSWINDOWS)}
   avformatLibName = 'avformat-59.dll';
{$ELSEIF defined(LINUX)}
   avformatLibName = 'libavformat.so.59';
{$ELSEIF defined(DARWIN)}
   avformatLibName = 'libavformat.59.dylib';
{$ELSE}
   avformatLibName = 'libavformat.so.59';
{$ENDIF}

Const
   AVIO_SEEKABLE_NORMAL = (1 Shl 0);
   AVIO_SEEKABLE_TIME = (1 Shl 1);
   AVSEEK_SIZE = $10000;
   AVSEEK_FORCE = $20000;
   AVIO_FLAG_READ = 1;
   AVIO_FLAG_WRITE = 2;
   AVIO_FLAG_READ_WRITE = (AVIO_FLAG_READ Or AVIO_FLAG_WRITE);
   AVIO_FLAG_NONBLOCK = 8;
   AVIO_FLAG_DIRECT = $8000;
   AVFMT_FLAG_GENPTS = $0001;
   AVFMT_FLAG_IGNIDX = $0002;
   AVFMT_FLAG_NONBLOCK = $0004;
   AVFMT_FLAG_IGNDTS = $0008;
   AVFMT_FLAG_NOFILLIN = $0010;
   AVFMT_FLAG_NOPARSE = $0020;
   AVFMT_FLAG_NOBUFFER = $0040;
   AVFMT_FLAG_CUSTOM_IO = $0080;
   AVFMT_FLAG_DISCARD_CORRUPT = $0100;
   AVFMT_FLAG_FLUSH_PACKETS = $0200;
   AVFMT_FLAG_BITEXACT = $0400;
   AVFMT_FLAG_MP4A_LATM = $8000;
   AVFMT_FLAG_SORT_DTS = $10000;
   AVFMT_FLAG_PRIV_OPT = $20000;
   AVFMT_FLAG_KEEP_SIDE_DATA = $40000;
   AVFMT_FLAG_FAST_SEEK = $80000;
   AVFMT_FLAG_SHORTEST = $100000;
   AVFMT_FLAG_AUTO_BSF = $200000;
   FF_FDEBUG_TS = $0001;
   AVFMT_EVENT_FLAG_METADATA_UPDATED = $0001;
   AVFMT_AVOID_NEG_TS_AUTO = -1;
   AVFMT_AVOID_NEG_TS_MAKE_NON_NEGATIVE = 1;
   AVFMT_AVOID_NEG_TS_MAKE_ZERO = 2;
   AV_PROGRAM_RUNNING = 1;
   AVFMTCTX_NOHEADER = $0001;
   AVFMTCTX_UNSEEKABLE = $0002;
   AVPROBE_SCORE_EXTENSION = 50;
   AVPROBE_SCORE_MIME = 75;
   AVPROBE_SCORE_MAX = 100;
   AVPROBE_PADDING_SIZE = 32;
   AVPROBE_SCORE_RETRY = (AVPROBE_SCORE_MAX Div 4);
   AVPROBE_SCORE_STREAM_RETRY = (AVPROBE_SCORE_MAX Div 4 - 1);
   AVFMT_NOFILE = $0001;
   AVFMT_NEEDNUMBER = $0002;
   AVFMT_SHOW_IDS = $0008;
   AVFMT_GLOBALHEADER = $0040;
   AVFMT_NOTIMESTAMPS = $0080;
   AVFMT_GENERIC_INDEX = $0100;
   AVFMT_TS_DISCONT = $0200;
   AVFMT_VARIABLE_FPS = $0400;
   AVFMT_NODIMENSIONS = $0800;
   AVFMT_NOSTREAMS = $1000;
   AVFMT_NOBINSEARCH = $2000;
   AVFMT_NOGENSEARCH = $4000;
   AVFMT_NO_BYTE_SEEK = $8000;
   AVFMT_ALLOW_FLUSH = $10000;
   AVFMT_TS_NONSTRICT = $20000;
   AVFMT_TS_NEGATIVE = $40000;
   AVFMT_SEEK_TO_PTS = $4000000;
   AVINDEX_KEYFRAME = $0001;
   AVINDEX_DISCARD_FRAME = $0002;
   AV_DISPOSITION_DEFAULT = $0001;
   AV_DISPOSITION_DUB = $0002;
   AV_DISPOSITION_ORIGINAL = $0004;
   AV_DISPOSITION_COMMENT = $0008;
   AV_DISPOSITION_LYRICS = $0010;
   AV_DISPOSITION_KARAOKE = $0020;
   AV_DISPOSITION_FORCED = $0040;
   AV_DISPOSITION_HEARING_IMPAIRED = $0080;
   AV_DISPOSITION_VISUAL_IMPAIRED = $0100;
   AV_DISPOSITION_CLEAN_EFFECTS = $0200;
   AV_DISPOSITION_ATTACHED_PIC = $0400;
   AV_DISPOSITION_TIMED_THUMBNAILS = $0800;
   AV_DISPOSITION_CAPTIONS = $10000;
   AV_DISPOSITION_DESCRIPTIONS = $20000;
   AV_DISPOSITION_METADATA = $40000;
   AV_DISPOSITION_DEPENDENT = $80000;
   AV_DISPOSITION_STILL_IMAGE = $100000;
   AV_PTS_WRAP_IGNORE = 0;
   AV_PTS_WRAP_ADD_OFFSET = 1;
   AV_PTS_WRAP_SUB_OFFSET = -1;
   AVSTREAM_EVENT_FLAG_METADATA_UPDATED = $0001;
   MAX_STD_TIMEBASES = (30 * 12 + 30 + 3 + 6);
   MAX_REORDER_DELAY = 16;
   AVSEEK_FLAG_BACKWARD = 1;
   AVSEEK_FLAG_BYTE = 2;
   AVSEEK_FLAG_ANY = 4;
   AVSEEK_FLAG_FRAME = 8;
   AVSTREAM_INIT_IN_WRITE_HEADER = 0;
   AVSTREAM_INIT_IN_INIT_OUTPUT = 1;
   AV_FRAME_FILENAME_FLAGS_MULTIPLE = 1;

Type
   pAVFilterFormats = ^AVFilterFormats;
   ppAVFilterFormats = ^pAVFilterFormats;
   pppAVFilterFormats = ^ppAVFilterFormats;

   AVFilterFormats = Record
      nb_formats: unsigned;
      formats: pint;
      refcount: unsigned;
      refs: pppAVFilterFormats;
   End;

   pAVFilterChannelLayouts = ^AVFilterChannelLayouts;
   ppAVFilterChannelLayouts = ^pAVFilterChannelLayouts;
   pppAVFilterChannelLayouts = ^ppAVFilterChannelLayouts;
   AVFilterChannelLayouts = Record
      channel_layouts: puint64_t;
      nb_channel_layouts: int;
      all_layouts: AnsiChar;
      all_counts: AnsiChar;
      refcount: unsigned;
      refs: pppAVFilterChannelLayouts;
   End;

   pAVIOInterruptCB = ^AVIOInterruptCB;
   AVIOInterruptCB = Record
      callback: Function(p: pointer): int; cdecl;
      opaque: pointer;
   End;
   TAVIOInterruptCB =AVIOInterruptCB;

   AVIODirEntryType = (
       AVIO_ENTRY_UNKNOWN,
       AVIO_ENTRY_BLOCK_DEVICE,
       AVIO_ENTRY_CHARACTER_DEVICE,
       AVIO_ENTRY_DIRECTORY,
       AVIO_ENTRY_NAMED_PIPE,
       AVIO_ENTRY_SYMBOLIC_LINK,
       AVIO_ENTRY_SOCKET,
       AVIO_ENTRY_FILE,
       AVIO_ENTRY_SERVER,
       AVIO_ENTRY_SHARE,
       AVIO_ENTRY_WORKGROUP
       );
   TAVIODirEntryType = AVIODirEntryType;

   pAVIODirEntry = ^AVIODirEntry;
   AVIODirEntry = Record
      name: PAnsiChar;
      _type: int;
      utf8: int;
      size: int64_t;
      modification_timestamp: int64_t;
      access_timestamp: int64_t;
      status_change_timestamp: int64_t;
      user_id: int64_t;
      group_id: int64_t;
      filemode: int64_t;
   End;

   pURLContext = ^URLContext;
   URLContext = Record
   End;

   pAVIODirContext = ^AVIODirContext;
   AVIODirContext = Record
      url_context: pURLContext;
   End;

   AVIODataMarkerType = (
      AVIO_DATA_MARKER_HEADER,
      AVIO_DATA_MARKER_SYNC_POINT,
      AVIO_DATA_MARKER_BOUNDARY_POINT,
      AVIO_DATA_MARKER_UNKNOWN,
      AVIO_DATA_MARKER_TRAILER,
      AVIO_DATA_MARKER_FLUSH_POINT);

   pAVIOContext = ^AVIOContext;
   AVIOContext = Record
      av_class: pAVClass;
      buffer: punsignedchar;
      buffer_size: int;
      buf_ptr: punsignedchar;
      buf_end: punsignedchar;
      opaque: pointer;
      read_packet: Function(opaque: pointer; buf: puint8_t; buf_size: int): int; cdecl;
      write_packet: Function(opaque: pointer; buf: puint8_t; buf_size: int): int; cdecl;
      seek: Function(opaque: pointer; offset: int64_t; whence: int): int64_t; cdecl;
      pos: int64_t;
      eof_reached: int;
      write_flag: int;
      max_packet_size: int;
      checksum: unsigned_long;
      checksum_ptr: punsignedchar;
      update_checksum: Function(checksum: unsigned_long; Const buf: puint8_t; size: unsigned_int): unsigned_long; cdecl;
      error: int;
      read_pause: Function(opaque: pointer; pause: int): int; cdecl;
      read_seek: Function(opaque: pointer; stream_index: int; timestamp: int64_t; flags: int): int64_t; cdecl;
      seekable: int;
      maxsize: int64_t;
      direct: int;
      bytes_read: int64_t;
      seek_count: int;
      writeout_count: int;
      orig_buffer_size: int;
      short_seek_threshold: int;
      protocol_whitelist: PAnsiChar;
      protocol_blacklist: PAnsiChar;
      write_data_type: Function(opaque: pointer; buf: puint8_t; buf_size: int; _type: AVIODataMarkerType; time: int64_t): int; cdecl;
      ignore_boundary_point: int;
      current_type: AVIODataMarkerType;
      last_time: int64_t;
      short_seek_get: Function(opaque: pointer): int; cdecl;
      written: int64_t;
      buf_ptr_max: punsigned_char;
      min_packet_size: int;
   End;

   Tavio_alloc_context_read_packet = Function(opaque: pointer; buf: puint8_t; buf_size: int): int; cdecl;
   Tavio_alloc_context_write_packet = Tavio_alloc_context_read_packet;
   Tavio_alloc_context_seek = Function(opaque: pointer; offset: int64_t; whence: int): int64_t; cdecl;

   pAVFormatContext = ^AVFormatContext;
   ppAVFormatContext = ^pAVFormatContext;
   pAVFormatInternal = ^AVFormatInternal;
   pAVInputFormat = ^AVInputFormat;
   pAVCodecTag = ^AVCodecTag;
   ppAVCodecTag = ^pAVCodecTag;
   pAVProbeData = ^AVProbeData;
   pAVDeviceInfoList = ^AVDeviceInfoList;
   pAVDeviceCapabilitiesQuery = ^AVDeviceCapabilitiesQuery;
   pAVOutputFormat = ^AVOutputFormat;
   pAVChapter = ^AVChapter;
   ppAVChapter = ^pAVChapter;
   pAVStream = ^AVStream;
   ppAVStream = ^pAVStream;
   pAVProgram = ^AVProgram;
   ppAVProgram = ^pAVProgram;

   AVChapter = Record
      id: int;
      time_base: AVRational;
      start, _end: int64_t;
      metadata: pAVDictionary;
   End;

   Tav_format_control_message = Function(s: pAVFormatContext; _type: int; data: pointer; data_size: size_t): int; cdecl;
   TAVOpenCallback = Function(s: pAVFormatContext; Var pb: pAVIOContext; Const url: PAnsiChar; flags: int; Const int_cb: pAVIOInterruptCB; Var options: pAVDictionary): int; cdecl;

   AVDurationEstimationMethod = (
      AVFMT_DURATION_FROM_PTS,
      AVFMT_DURATION_FROM_STREAM,
      AVFMT_DURATION_FROM_BITRATE
      );

   AVFormatInternal = Record
   End;

   AVInputFormat = Record
      name: PAnsiChar;
      long_name: PAnsiChar;
      flags: int;
      extensions: PAnsiChar;
      codec_tag: ppAVCodecTag;
      priv_class: pAVClass;
      mime_type: PAnsiChar;
      next: pAVInputFormat;
      raw_codec_id: int;
      priv_data_size: int;
      read_probe: Function(Const p: pAVProbeData): int; cdecl;
      read_header: Function(p: pAVFormatContext): int; cdecl;
      read_packet: Function(p: pAVFormatContext; pkt: pAVPacket): int; cdecl;
      read_close: Function(p: pAVFormatContext): int; cdecl;
      read_seek: Function(p: pAVFormatContext; stream_index: int; timestamp: int64_t; flags: int): int; cdecl;
      read_timestamp: Function(s: pAVFormatContext; stream_index: int; Var pos: int64_t; pos_limit: int64_t): int64_t; cdecl;
      read_play: Function(p: pAVFormatContext): int; cdecl;
      read_pause: Function(p: pAVFormatContext): int; cdecl;
      read_seek2: Function(s: pAVFormatContext; stream_index: int; min_ts: int64_t; ts: int64_t; max_ts: int64_t; flags: int): int; cdecl;
      get_device_list: Function(s: pAVFormatContext; device_list: pAVDeviceInfoList): int; cdecl;
      create_device_capabilities: Function(s: pAVFormatContext; caps: pAVDeviceCapabilitiesQuery): int; cdecl;
      free_device_capabilities: Function(s: pAVFormatContext; caps: pAVDeviceCapabilitiesQuery): int; cdecl;
   End;

   AVFormatContext = Record
      av_class: pAVClass;
      iformat: pAVInputFormat;
      oformat: pAVOutputFormat;
      priv_data: pointer;
      pb: pAVIOContext;
      ctx_flags: int;
      nb_streams: unsigned_int;
      streams: ppAVStream;
      filename: Array[0..1024 - 1] Of AnsiChar;
      url: PAnsiChar;
      start_time: int64_t;
      duration: int64_t;
      bit_rate: int64_t;
      packet_size: unsigned_int;
      max_delay: int;
      flags: int;
      robesize: int64_t;
      ax_analyze_duration: int64_t;
      key: puint8_t;
      keylen: int;
      nb_programs: unsigned_int;
      programs: ppAVProgram;
      video_codec_id: AVCodecID;
      audio_codec_id: AVCodecID;
      subtitle_codec_id: AVCodecID;
      max_index_size: unsigned_int;
      max_picture_buffer: unsigned_int;
      nb_chapters: unsigned_int;
      chapters: ppAVChapter;
      metadata: pAVDictionary;
      start_time_realtime: int64_t;
      fps_probe_size: int;
      error_recognition: int;
      interrupt_callback: AVIOInterruptCB;
      debug: int;
      max_interleave_delta: int64_t;
      strict_std_compliance: int;
      event_flags: int;
      max_ts_probe: int;
      avoid_negative_ts: int;
      ts_id: int;
      audio_preload: int;
      max_chunk_duration: int;
      max_chunk_size: int;
      use_wallclock_as_timestamps: int;
      avio_flags: int;
      duration_estimation_method: AVDurationEstimationMethod;
      skip_initial_bytes: int64_t;
      correct_ts_overflow: unsigned_int;
      seek2any: int;
      flush_packets: int;
      probe_score: int;
      format_probesize: int;
      codec_whitelist: PAnsiChar;
      format_whitelist: PAnsiChar;
      internal: pAVFormatInternal;
      io_repositioned: int;
      video_codec: pAVCodec;
      audio_codec: pAVCodec;
      subtitle_codec: pAVCodec;
      data_codec: pAVCodec;
      metadata_header_padding: int;
      opaque: pointer;
      control_message_cb: Tav_format_control_message;
      output_ts_offset: int64_t;
      dump_separator: puint8_t;
      data_codec_id: AVCodecID;
      open_cb: Function(s: pAVFormatContext; Var p: pAVIOContext; Const url: PAnsiChar; flags: int; Const int_cb: pAVIOInterruptCB; Var options: pAVDictionary): int; cdecl;
      protocol_whitelist: PAnsiChar;
      io_open: Function(s: pAVFormatContext; Var pb: pAVIOContext; Const url: PAnsiChar; flags: int; Var options: pAVDictionary): int; cdecl;
      io_close: Procedure(s: pAVFormatContext; pb: pAVIOContext); cdecl;
      protocol_blacklist: PAnsiChar;
      max_streams: int;
      skip_estimate_duration_from_pts: int;
   End;

   AVCodecTag = Record
   End;

   AVProbeData = Record
      filename: PAnsiChar;
      buf: punsigned_char;
      buf_size: int;
      mime_type: PAnsiChar;
   End;

   AVDeviceInfoList = Record
   End;

   AVDeviceCapabilitiesQuery = Record
   End;

   AVOutputFormat = Record
      name: PAnsiChar;
      long_name: PAnsiChar;
      mime_type: PAnsiChar;
      extensions: PAnsiChar;
      audio_codec: AVCodecID;
      video_codec: AVCodecID;
      subtitle_codec: AVCodecID;
      flags: int;
      codec_tag: ppAVCodecTag;
      priv_class: pAVClass;
      next: pAVOutputFormat;
      priv_data_size: int;
      write_header: Function(p: pAVFormatContext): int; cdecl;
      write_packet: Function(fc: pAVFormatContext; pkt: pAVPacket): int; cdecl;
      write_trailer: Function(p: pAVFormatContext): int; cdecl;
      interleave_packet: Function(p: pAVFormatContext; _out: pAVPacket; _in: pAVPacket; flush: int): int; cdecl;
      query_codec: Function(id: AVCodecID; std_compliance: int): int; cdecl;
      get_output_timestamp: Procedure(s: pAVFormatContext; stream: int; Var dts: int64_t; Var wall: int64_t); cdecl;
      control_message: Function(s: pAVFormatContext; _type: int; data: pointer; data_size: size_t): int; cdecl;
      write_uncoded_frame: Function(p: pAVFormatContext; stream_index: int; Var frame: pAVFrame; flags: unsigned): int; cdecl;
      get_device_list: Function(s: pAVFormatContext; device_list: pAVDeviceInfoList): int; cdecl;
      create_device_capabilities: Function(s: pAVFormatContext; caps: pAVDeviceCapabilitiesQuery): int; cdecl;
      free_device_capabilities: Function(s: pAVFormatContext; caps: pAVDeviceCapabilitiesQuery): int; cdecl;
      data_codec: AVCodecID;
      init: Function(p: pAVFormatContext): int; cdecl;
      deinit: Procedure(p: pAVFormatContext); cdecl;
      check_bitstream: Function(p: pAVFormatContext; Const pkt: pAVPacket): int; cdecl;
   End;

   AVStreamParseType = (
      AVSTREAM_PARSE_NONE, AVSTREAM_PARSE_FULL,
      AVSTREAM_PARSE_HEADERS,
      AVSTREAM_PARSE_TIMESTAMPS,
      AVSTREAM_PARSE_FULL_ONCE,
      AVSTREAM_PARSE_FULL_RAW
      );

   pAVIndexEntry = ^AVIndexEntry;
   AVIndexEntry = Record
      pos: int64_t;
      timestamp: int64_t;
      flag_size: int32;
      min_distance: int;
   End;

   pAVStreamInternal = ^AVStreamInternal;
   AVStreamInternal = Record
   End;

   Tduration_error = Array[0..1, 0..MAX_STD_TIMEBASES] Of Double;
   pduration_error = ^Tduration_error;

   pAVStream_info = ^AVStream_info;
   AVStream_info = Record
      last_dts: int64_t;
      duration_gcd: int64_t;
      duration_count: int;
      rfps_duration_sum: int64_t;
      duration_error: pduration_error;
      codec_info_duration: int64_t;
      codec_info_duration_fields: int64_t;
      frame_delay_evidence: int;
      found_decoder: int;
      last_duration: int64_t;
      fps_first_dts: int64_t;
      fps_first_dts_idx: int;
      fps_last_dts: int64_t;
      fps_last_dts_idx: int;
   End;

   Tpts_buffer_int64_t = Array[0..MAX_REORDER_DELAY] Of int64_t;
   Tpts_reorder_error_count_uint8_t = Array[0..MAX_REORDER_DELAY] Of uint8_t;

   pAVPacketList = ^AVPacketList;
   AVPacketList = Record
      pkt: AVPacket;
      next: pAVPacketList;
   End;
   TAVPacketList =AVPacketList ;

   AVStream = Record
      index: int;
      id: int;
      codec: pAVCodecContext;
      priv_data: pointer;
      time_base: AVRational;
      start_time: int64_t;
      duration: int64_t;
      nb_frames: int64_t;
      disposition: int;
      discard: AVDiscard;
      sample_aspect_ratio: AVRational;
      metadata: pAVDictionary;
      avg_frame_rate: AVRational;
      attached_pic: AVPacket;
      side_data: pAVPacketSideData;
      nb_side_data: int;
      event_flags: int;
      r_frame_rate: AVRational;
      recommended_encoder_configuration: PAnsiChar;
      codecpar: pAVCodecParameters;
      info: pAVStream_info;
      pts_wrap_bits: int;
      first_dts: int64_t;
      cur_dts: int64_t;
      last_IP_pts: int64_t;
      last_IP_duration: int;
      probe_packets: int;
      codec_info_nb_frames: int;
      need_parsing: AVStreamParseType;
      parser: pAVCodecParserContext;
      last_in_packet_buffer: pAVPacketList;
      probe_data: AVProbeData;
      pts_buffer: Tpts_buffer_int64_t;
      index_entries: pAVIndexEntry;
      nb_index_entries: int;
      index_entries_allocated_size: unsigned_int;
      stream_identifier: int;
      program_num: int;
      pmt_version: int;
      pmt_stream_idx: int;
      interleaver_chunk_size: int64_t;
      interleaver_chunk_duration: int64_t;
      request_probe: int;
      skip_to_keyframe: int;
      skip_samples: int;
      start_skip_samples: int64_t;
      first_discard_sample: int64_t;
      last_discard_sample: int64_t;
      nb_decoded_frames: int;
      mux_ts_offset: int64_t;
      pts_wrap_reference: int64_t;
      pts_wrap_behavior: int;
      update_initial_durations_done: int;
      pts_reorder_error: Tpts_buffer_int64_t;
      pts_reorder_error_count: Tpts_reorder_error_count_uint8_t;
      last_dts_for_order_check: int64_t;
      dts_ordered: uint8_t;
      dts_misordered: uint8_t;
      inject_global_side_data: int;
      display_aspect_ratio: AVRational;
      internal: pAVStreamInternal;
   End;

   AVProgram = Record
      id: int;
      flags: int;
      discard: AVDiscard;
      stream_index: punsigned_int;
      nb_stream_indexes: unsigned_int;
      metadata: pAVDictionary;
      program_num: int;
      pmt_pid: int;
      pcr_pid: int;
      pmt_version: int;
      start_time: int64_t;
      end_time: int64_t;
      pts_wrap_reference: int64_t;
      pts_wrap_behavior: int;
   End;

   AVTimebaseSource = (
      AVFMT_TBCF_AUTO = -1,
      AVFMT_TBCF_DECODER,
      AVFMT_TBCF_DEMUXER
      );
   
 //-------------------------------------------------
 Type

   Tavio_find_protocol_name = Function(Const url: PAnsiChar): PAnsiChar; cdecl;
   Tavio_check = Function(Const url: PAnsiChar; flags: int): int; cdecl;
   Tavpriv_io_move = Function(Const url_src: PAnsiChar; Const url_dst: PAnsiChar): int; cdecl;
   Tavpriv_io_delete = Function(Const url: PAnsiChar): int; cdecl;
   Tavio_open_dir = Function(Var s: pAVIODirContext; Const url: PAnsiChar; options: ppAVDictionary): int; cdecl;
   Tavio_read_dir = Function(s: pAVIODirContext; Var next: pAVIODirEntry): int; cdecl;
   Tavio_close_dir = Function(Var s: pAVIODirContext): int; cdecl;
   Tavio_free_directory_entry = Procedure(Var entry: pAVIODirEntry); cdecl;
   Tavio_alloc_context = Function(buffer: punsigned_char; buffer_size: int; write_flag: int; opaque: pointer; read_packet: Tavio_alloc_context_read_packet;
                                  write_packet: Tavio_alloc_context_write_packet; seek: Tavio_alloc_context_seek): pAVIOContext; cdecl;
   Tavio_context_free = Procedure(Var s: pAVIOContext); cdecl;
   Tavio_w8 = Procedure(s: pAVIOContext; b: int); cdecl;
   Tavio_write = Procedure(s: pAVIOContext; Const buf: punsigned_char; size: int); cdecl;
   Tavio_wl64 = Procedure(s: pAVIOContext; val: uint64_t); cdecl;
   Tavio_wb64 = Procedure(s: pAVIOContext; val: uint64_t); cdecl;
   Tavio_wl32 = Procedure(s: pAVIOContext; val: unsigned_int); cdecl;
   Tavio_wb32 = Procedure(s: pAVIOContext; val: unsigned_int); cdecl;
   Tavio_wl24 = Procedure(s: pAVIOContext; val: unsigned_int); cdecl;
   Tavio_wb24 = Procedure(s: pAVIOContext; val: unsigned_int); cdecl;
   Tavio_wl16 = Procedure(s: pAVIOContext; val: unsigned_int); cdecl;
   Tavio_wb16 = Procedure(s: pAVIOContext; val: unsigned_int); cdecl;
   Tavio_put_str = Function(s: pAVIOContext; Const str: PAnsiChar): int; cdecl;
   Tavio_put_str16le = Function(s: pAVIOContext; Const str: PAnsiChar): int; cdecl;
   Tavio_put_str16be = Function(s: pAVIOContext; Const str: PAnsiChar): int; cdecl;
   Tavio_write_marker = Procedure(s: pAVIOContext; time: int64_t; _type: AVIODataMarkerType); cdecl;
   Tavio_seek = Function(s: pAVIOContext; offset: int64_t; whence: int): int64_t; cdecl;
   Tavio_skip = Function(s: pAVIOContext; offset: int64_t): int64_t; cdecl;
   Tavio_size = Function(s: pAVIOContext): int64_t; cdecl;
   Tavio_feof = Function(s: pAVIOContext): int; cdecl;
   Tavio_flush = Procedure(s: pAVIOContext); cdecl;
   Tavio_read = Function(s: pAVIOContext; buf: punsigned_char; size: int): int; cdecl;
   Tavio_read_partial = Function(s: pAVIOContext; buf: punsigned_char; size: int): int; cdecl;
   Tavio_r8 = Function(s: pAVIOContext): int; cdecl;
   Tavio_rl16 = Function(s: pAVIOContext): unsigned_int; cdecl;
   Tavio_rl24 = Function(s: pAVIOContext): unsigned_int; cdecl;
   Tavio_rl32 = Function(s: pAVIOContext): unsigned_int; cdecl;
   Tavio_rl64 = Function(s: pAVIOContext): uint64_t; cdecl;
   Tavio_rb16 = Function(s: pAVIOContext): unsigned_int; cdecl;
   Tavio_rb24 = Function(s: pAVIOContext): unsigned_int; cdecl;
   Tavio_rb32 = Function(s: pAVIOContext): unsigned_int; cdecl;
   Tavio_rb64 = Function(s: pAVIOContext): uint64_t; cdecl;
   Tavio_get_str = Function(pb: pAVIOContext; maxlen: int; buf: PAnsiChar; buflen: int): int; cdecl;
   Tavio_get_str16le = Function(pb: pAVIOContext; maxlen: int; buf: PAnsiChar; buflen: int): int; cdecl;
   Tavio_get_str16be = Function(pb: pAVIOContext; maxlen: int; buf: PAnsiChar; buflen: int): int; cdecl;
   Tavio_open = Function(Var s: pAVIOContext; Const url: PAnsiChar; flags: int): int; cdecl;
   Tavio_open2 = Function(Var s: pAVIOContext; Const url: PAnsiChar; flags: int; Const int_cb: pAVIOInterruptCB; Var options: pAVDictionary): int; cdecl;
   Tavio_close = Function(s: pAVIOContext): int; cdecl;
   Tavio_closep = Function(Var s: pAVIOContext): int; cdecl;
   Tavio_open_dyn_buf = Function(Var s: pAVIOContext): int; cdecl;
   Tavio_get_dyn_buf = Function(s: pAVIOContext; Var pbuffer: puint8_t): int; cdecl;
   Tavio_close_dyn_buf = Function(s: pAVIOContext; Var pbuffer: puint8_t): int; cdecl;
   Tavio_enum_protocols = Function(Var opaque: pointer; output: int): PAnsiChar; cdecl;
   Tavio_pause = Function(h: pAVIOContext; pause: int): int; cdecl;
   Tavio_seek_time = Function(h: pAVIOContext; stream_index: int; timestamp: int64_t; flags: int): int64_t; cdecl;
   Tavio_read_to_bprint = Function(h: pAVIOContext; pb: pAVBPrint; max_size: size_t): int; cdecl;
   Tavio_accept = Function(s: pAVIOContext; Var c: pAVIOContext): int; cdecl;
   Tavio_handshake = Function(c: pAVIOContext): int; cdecl;
   Tav_get_packet = Function(s: pAVIOContext; pkt: pAVPacket; size: int): int; cdecl;
   Tav_append_packet = Function(s: pAVIOContext; pkt: pAVPacket; size: int): int; cdecl;
   Tav_stream_get_r_frame_rate = Function(Const s: pAVStream): AVRational; cdecl;
   Tav_stream_set_r_frame_rate = Procedure(s: pAVStream; r: AVRational); cdecl;
   Tav_stream_get_recommended_encoder_configuration = Function(Const s: pAVStream): PAnsiChar; cdecl;
   Tav_stream_set_recommended_encoder_configuration = Procedure(s: pAVStream; configuration: PAnsiChar); cdecl;
   Tav_stream_get_parser = Function(Const s: pAVStream): pAVCodecParserContext; cdecl;
   Tav_stream_get_end_pts = Function(Const st: pAVStream): int64_t; cdecl;
   Tav_format_get_probe_score = Function(Const s: pAVFormatContext): int; cdecl;
   Tav_format_get_video_codec = Function(Const s: pAVFormatContext): pAVCodec; cdecl;
   Tav_format_set_video_codec = Procedure(s: pAVFormatContext; c: pAVCodec); cdecl;
   Tav_format_get_audio_codec = Function(Const s: pAVFormatContext): pAVCodec; cdecl;
   Tav_format_set_audio_codec = Procedure(s: pAVFormatContext; c: pAVCodec); cdecl;
   Tav_format_get_subtitle_codec = Function(Const s: pAVFormatContext): pAVCodec; cdecl;
   Tav_format_set_subtitle_codec = Procedure(s: pAVFormatContext; c: pAVCodec); cdecl;
   Tav_format_get_data_codec = Function(Const s: pAVFormatContext): pAVCodec; cdecl;
   Tav_format_set_data_codec = Procedure(s: pAVFormatContext; c: pAVCodec); cdecl;
   Tav_format_get_metadata_header_padding = Function(Const s: pAVFormatContext): int; cdecl;
   Tav_format_set_metadata_header_padding = Procedure(s: pAVFormatContext; c: int); cdecl;
   Tav_format_get_opaque = Function(Const s: pAVFormatContext): pointer; cdecl;
   Tav_format_set_opaque = Procedure(s: pAVFormatContext; opaque: pointer); cdecl;
   Tav_format_get_control_message_cb = Function(Const s: pAVFormatContext): Tav_format_control_message; cdecl;
   Tav_format_set_control_message_cb = Procedure(s: pAVFormatContext; callback: Tav_format_control_message); cdecl;
   Tav_format_get_open_cb = Function(Const s: pAVFormatContext): TAVOpenCallback; cdecl;
   Tav_format_set_open_cb = Procedure(s: pAVFormatContext; callback: TAVOpenCallback); cdecl;
   Tav_format_inject_global_side_data = Procedure(s: pAVFormatContext); cdecl;
   Tav_fmt_ctx_get_duration_estimation_method = Function(Const ctx: pAVFormatContext): AVDurationEstimationMethod; cdecl;
   Tavformat_version = Function(): unsigned; cdecl;
   Tavformat_configuration = Function(): PAnsiChar; cdecl;
   Tavformat_license = Function(): PAnsiChar; cdecl;
   Tav_register_all = Procedure(); cdecl;
   Tav_register_input_format = Procedure(format: pAVInputFormat); cdecl;
   Tav_register_output_format = Procedure(format: pAVOutputFormat); cdecl;
   Tavformat_network_init = Function(): int; cdecl;
   Tavformat_network_deinit = Function(): int; cdecl;
   Tav_iformat_next = Function(Const f: pAVInputFormat): pAVInputFormat; cdecl;
   Tav_oformat_next = Function(Const f: pAVOutputFormat): pAVOutputFormat; cdecl;
   Tav_muxer_iterate = Function(Var opaque: pointer): pAVOutputFormat; cdecl;
   Tav_demuxer_iterate = Function(Var opaque: pointer): pAVInputFormat; cdecl;
   Tavformat_alloc_context = Function(): pAVFormatContext; cdecl;
   Tavformat_free_context = Procedure(s: pAVFormatContext); cdecl;
   Tavformat_get_class = Function(): pAVClass; cdecl;
   Tavformat_new_stream = Function(s: pAVFormatContext; Const c: pAVCodec): pAVStream; cdecl;
   Tav_stream_add_side_data = Function(st: pAVStream; _type: AVPacketSideDataType; data: puint8_t; size: size_t): int; cdecl;
   Tav_stream_new_side_data = Function(stream: pAVStream; _type: AVPacketSideDataType; size: int): puint8_t; cdecl;
   Tav_stream_get_side_data = Function(Const stream: pAVStream; _type: AVPacketSideDataType; Var size: int): puint8_t; cdecl;
   Tav_new_program = Function(s: pAVFormatContext; id: int): pAVProgram; cdecl;
   Tavformat_alloc_output_context2 = Function(Var ctx: pAVFormatContext; oformat: pAVOutputFormat; Const format_name: PAnsiChar;
                                              Const filename: PAnsiChar): int; cdecl;
   Tav_find_input_format = Function(Const short_name: PAnsiChar): pAVInputFormat; cdecl;
   Tav_probe_input_format = Function(pd: pAVProbeData; is_opened: int): pAVInputFormat; cdecl;
   Tav_probe_input_format2 = Function(pd: pAVProbeData; is_opened: int; Var score_max: int): pAVInputFormat; cdecl;
   Tav_probe_input_format3 = Function(pd: pAVProbeData; is_opened: int; Var score_ret: int): pAVInputFormat; cdecl;
   Tav_probe_input_buffer2 = Function(pb: pAVIOContext; Var fmt: pAVInputFormat; Const url: PAnsiChar; logctx: pointer;
                                      offset: unsigned_int; max_probe_size: unsigned_int): int; cdecl;
   Tav_probe_input_buffer = Function(pb: pAVIOContext; Var fmt: pAVInputFormat; Const url: PAnsiChar; logctx: pointer;
                                     offset: unsigned_int; max_probe_size: unsigned_int): int; cdecl;
   Tavformat_open_input = Function(Var ps: pAVFormatContext; Const url: PAnsiChar; fmt: pAVInputFormat; options: pAVDictionary): int; cdecl;
   Tav_demuxer_open = Function(ic: pAVFormatContext): int; cdecl;
   Tavformat_find_stream_info = Function(ic: pAVFormatContext; options: pAVDictionary): int; cdecl;
   Tav_find_program_from_stream = Function(ic: pAVFormatContext; last: pAVProgram; s: int): pAVProgram; cdecl;
   Tav_program_add_stream_index = Procedure(ac: pAVFormatContext; progid: int; idx: unsigned_int); cdecl;
   Tav_find_best_stream = Function(ic: pAVFormatContext; _type: AVMediaType; wanted_stream_nb: int; related_stream: int;decoder_ret: pAVCodec; flags: int): int; cdecl;
   Tav_read_frame = Function(s: pAVFormatContext; Var pkt: AVPacket): int; cdecl;
   Tav_seek_frame = Function(s: pAVFormatContext; stream_index: int; timestamp: int64_t; flags: int): int; cdecl;
   Tavformat_seek_file = Function(s: pAVFormatContext; stream_index: int; min_ts: int64_t; ts: int64_t; max_ts: int64_t; flags: int): int; cdecl;
   Tavformat_flush = Function(s: pAVFormatContext): int; cdecl;
   Tav_read_play = Function(s: pAVFormatContext): int; cdecl;
   Tav_read_pause = Function(s: pAVFormatContext): int; cdecl;
   Tavformat_close_input = Procedure(Var s: pAVFormatContext); cdecl;
   Tavformat_write_header = Function(s: pAVFormatContext; options: ppAVDictionary): int; cdecl;
   Tavformat_init_output = Function(s: pAVFormatContext; Var options: pAVDictionary): int; cdecl;
   Tav_write_frame = Function(s: pAVFormatContext; pkt: pAVPacket): int; cdecl;
   Tav_interleaved_write_frame = Function(s: pAVFormatContext; pkt: pAVPacket): int; cdecl;
   Tav_write_uncoded_frame = Function(s: pAVFormatContext; stream_index: int; frame: pAVFrame): int; cdecl;
   Tav_interleaved_write_uncoded_frame = Function(s: pAVFormatContext; stream_index: int; frame: pAVFrame): int; cdecl;
   Tav_write_uncoded_frame_query = Function(s: pAVFormatContext; stream_index: int): int; cdecl;
   Tav_write_trailer = Function(s: pAVFormatContext): int; cdecl;
   Tav_guess_format = Function(Const short_name: PAnsiChar; Const filename: PAnsiChar; Const mime_type: PAnsiChar): pAVOutputFormat; cdecl;
   Tav_guess_codec = Function(fmt: pAVOutputFormat; Const short_name: PAnsiChar; Const filename: PAnsiChar; Const mime_type: PAnsiChar;
                              _type: AVMediaType): AVCodecID; cdecl;
   Tav_get_output_timestamp = Function(s: pAVFormatContext; stream: int; Var dts: int64_t; Var wall: int64_t): int; cdecl;
   Tav_hex_dump = Procedure(f: pFILE; Const buf: puint8_t; size: int); cdecl;
   Tav_hex_dump_log = Procedure(avcl: pointer; level: int; Const buf: puint8_t; size: int); cdecl;
   Tav_pkt_dump2 = Procedure(f: pFILE; Const pkt: pAVPacket; dump_payload: int; Const st: pAVStream); cdecl;
   Tav_pkt_dump_log2 = Procedure(avcl: pointer; level: int; Const pkt: pAVPacket; dump_payload: int; Const st: pAVStream); cdecl;
   Tav_codec_get_id = Function(Const tags: ppAVCodecTag; tag: unsigned_int): AVCodecID; cdecl;
   Tav_codec_get_tag = Function(Const tags: ppAVCodecTag; id: AVCodecID): unsigned_int; cdecl;
   Tav_codec_get_tag2 = Function(Const tags: ppAVCodecTag; id: AVCodecID; tag: punsigned_int): int; cdecl;
   Tav_find_default_stream_index = Function(s: pAVFormatContext): int; cdecl;
   Tav_index_search_timestamp = Function(st: pAVStream; timestamp: int64_t; flags: int): int; cdecl;
   Tav_add_index_entry = Function(st: pAVStream; pos: int64_t; timestamp: int64_t; size: int; distance: int; flags: int): int; cdecl;
   Tav_url_split = Procedure(proto: PAnsiChar; proto_size: int; authorization: PAnsiChar; authorization_size: int; hostname: PAnsiChar; hostname_size: int;
                             Var port_ptr: int; path: PAnsiChar; path_size: int; Const url: PAnsiChar); cdecl;
   Tav_dump_format = Procedure(ic: pAVFormatContext; index: int; Const url: PAnsiChar; is_output: int); cdecl;
   Tav_get_frame_filename2 = Function(buf: PAnsiChar; buf_size: int; Const path: PAnsiChar; number: int; flags: int): int; cdecl;
   Tav_get_frame_filename = Function(buf: PAnsiChar; buf_size: int; Const path: PAnsiChar; number: int): int; cdecl;
   Tav_filename_number_test = Function(Const filename: PAnsiChar): int; cdecl;
   Tav_sdp_create = Function(ac: ppAVFormatContext; n_files: int; buf: PAnsiChar; size: int): int; cdecl;
   Tav_match_ext = Function(Const filename: PAnsiChar; Const extensions: PAnsiChar): int; cdecl;
   Tavformat_query_codec = Function(Const ofmt: pAVOutputFormat; codec_id: AVCodecID; std_compliance: int): int; cdecl;
   Tavformat_get_riff_video_tags = Function(): pAVCodecTag; cdecl;
   Tavformat_get_riff_audio_tags = Function(): pAVCodecTag; cdecl;
   Tavformat_get_mov_video_tags = Function(): pAVCodecTag; cdecl;
   Tavformat_get_mov_audio_tags = Function(): pAVCodecTag; cdecl;
   Tav_guess_sample_aspect_ratio = Function(format: pAVFormatContext; stream: pAVStream; frame: pAVFrame): AVRational; cdecl;
   Tav_guess_frame_rate = Function(ctx: pAVFormatContext; stream: pAVStream; frame: pAVFrame): AVRational; cdecl;
   Tavformat_match_stream_specifier = Function(s: pAVFormatContext; st: pAVStream; Const spec: PAnsiChar): int; cdecl;
   Tavformat_queue_attached_pictures = Function(s: pAVFormatContext): int; cdecl;
   Tav_apply_bitstream_filters = Function(codec: pAVCodecContext; pkt: pAVPacket; bsfc: pAVBitStreamFilterContext): int; cdecl;
   Tavformat_transfer_internal_stream_timing_info = Function(Const ofmt: pAVOutputFormat; ost: pAVStream; Const ist: pAVStream;
                                                             copy_tb: AVTimebaseSource): int; cdecl;
   Tav_stream_get_codec_timebase = Function(Const st: AVStream): AVRational; cdecl;   
   Tav_disposition_from_string = Function(Const disp: PAnsiChar): Integer; cdecl;
   Tav_disposition_to_string = Function(disposition: Integer): PAnsiChar; cdecl;   
   Tavpriv_set_pts_info = Procedure(s: PAVStream; pts_wrap_bits: Integer;pts_num, pts_den: Cardinal); cdecl;
   Tavformat_index_get_entries_count = Function(Const st: PAVStream): Integer; cdecl;
   Tavformat_index_get_entry = Function(st: PAVStream; idx: Integer): PAVIndexEntry; cdecl;
   Tavformat_index_get_entry_from_timestamp = Function(st: PAVStream;wanted_timestamp: Int64;flags: Integer): PAVIndexEntry; cdecl;   
   Tavio_printf = Function(s: PAVIOContext; Const fmt: PAnsiChar): Integer; cdecl;
   Tavio_protocol_get_class = Function(Const name: PAnsiChar): PAVClass; cdecl;

  //....................................................

Var

   avio_find_protocol_name : Tavio_find_protocol_name = Nil;
   avio_check : Tavio_check = Nil;
   avpriv_io_move : Tavpriv_io_move = Nil;
   avpriv_io_delete : Tavpriv_io_delete = Nil;
   avio_open_dir : Tavio_open_dir = Nil;
   avio_read_dir : Tavio_read_dir = Nil;
   avio_close_dir : Tavio_close_dir = Nil;
   avio_free_directory_entry : Tavio_free_directory_entry = Nil;
   avio_alloc_context : Tavio_alloc_context = Nil;
   avio_context_free : Tavio_context_free = Nil;
   avio_w8 : Tavio_w8 = Nil;
   avio_write : Tavio_write = Nil;
   avio_wl64 : Tavio_wl64 = Nil;
   avio_wb64 : Tavio_wb64 = Nil;
   avio_wl32 : Tavio_wl32 = Nil;
   avio_wb32 : Tavio_wb32 = Nil;
   avio_wl24 : Tavio_wl24 = Nil;
   avio_wb24 : Tavio_wb24 = Nil;
   avio_wl16 : Tavio_wl16 = Nil;
   avio_wb16 : Tavio_wb16 = Nil;
   avio_put_str : Tavio_put_str = Nil;
   avio_put_str16le : Tavio_put_str16le = Nil;
   avio_put_str16be : Tavio_put_str16be = Nil;
   avio_write_marker : Tavio_write_marker = Nil;
   avio_seek : Tavio_seek = Nil;
   avio_skip : Tavio_skip = Nil;
   avio_size : Tavio_size = Nil;
   avio_feof : Tavio_feof = Nil;
   avio_flush : Tavio_flush = Nil;
   avio_read : Tavio_read = Nil;
   avio_read_partial : Tavio_read_partial = Nil;
   avio_r8 : Tavio_r8 = Nil;
   avio_rl16 : Tavio_rl16 = Nil;
   avio_rl24 : Tavio_rl24 = Nil;
   avio_rl32 : Tavio_rl32 = Nil;
   avio_rl64 : Tavio_rl64 = Nil;
   avio_rb16 : Tavio_rb16 = Nil;
   avio_rb24 : Tavio_rb24 = Nil;
   avio_rb32 : Tavio_rb32 = Nil;
   avio_rb64 : Tavio_rb64 = Nil;
   avio_get_str : Tavio_get_str = Nil;
   avio_get_str16le : Tavio_get_str16le = Nil;
   avio_get_str16be : Tavio_get_str16be = Nil;
   avio_open : Tavio_open = Nil;
   avio_open2 : Tavio_open2 = Nil;
   avio_close : Tavio_close = Nil;
   avio_closep : Tavio_closep = Nil;
   avio_open_dyn_buf : Tavio_open_dyn_buf = Nil;
   avio_get_dyn_buf : Tavio_get_dyn_buf = Nil;
   avio_close_dyn_buf : Tavio_close_dyn_buf = Nil;
   avio_enum_protocols : Tavio_enum_protocols = Nil;
   avio_pause : Tavio_pause = Nil;
   avio_seek_time : Tavio_seek_time = Nil;
   avio_read_to_bprint : Tavio_read_to_bprint = Nil;
   avio_accept : Tavio_accept = Nil;
   avio_handshake : Tavio_handshake = Nil;
   av_get_packet : Tav_get_packet = Nil;
   av_append_packet : Tav_append_packet = Nil;
   av_stream_get_r_frame_rate : Tav_stream_get_r_frame_rate = Nil;
   av_stream_set_r_frame_rate : Tav_stream_set_r_frame_rate = Nil;
   av_stream_get_recommended_encoder_configuration : Tav_stream_get_recommended_encoder_configuration = Nil;
   av_stream_set_recommended_encoder_configuration : Tav_stream_set_recommended_encoder_configuration = Nil;
   av_stream_get_parser : Tav_stream_get_parser = Nil;
   av_stream_get_end_pts : Tav_stream_get_end_pts = Nil;
   av_format_get_probe_score : Tav_format_get_probe_score = Nil;
   av_format_get_video_codec : Tav_format_get_video_codec = Nil;
   av_format_set_video_codec : Tav_format_set_video_codec = Nil;
   av_format_get_audio_codec : Tav_format_get_audio_codec = Nil;
   av_format_set_audio_codec : Tav_format_set_audio_codec = Nil;
   av_format_get_subtitle_codec : Tav_format_get_subtitle_codec = Nil;
   av_format_set_subtitle_codec : Tav_format_set_subtitle_codec = Nil;
   av_format_get_data_codec : Tav_format_get_data_codec = Nil;
   av_format_set_data_codec : Tav_format_set_data_codec = Nil;
   av_format_get_metadata_header_padding : Tav_format_get_metadata_header_padding = Nil;
   av_format_set_metadata_header_padding : Tav_format_set_metadata_header_padding = Nil;
   av_format_get_opaque : Tav_format_get_opaque = Nil;
   av_format_set_opaque : Tav_format_set_opaque = Nil;
   av_format_get_control_message_cb : Tav_format_get_control_message_cb = Nil;
   av_format_set_control_message_cb : Tav_format_set_control_message_cb = Nil;
   av_format_get_open_cb : Tav_format_get_open_cb = Nil;
   av_format_set_open_cb : Tav_format_set_open_cb = Nil;
   av_format_inject_global_side_data : Tav_format_inject_global_side_data = Nil;
   av_fmt_ctx_get_duration_estimation_method : Tav_fmt_ctx_get_duration_estimation_method = Nil;
   avformat_version : Tavformat_version = Nil;
   avformat_configuration : Tavformat_configuration = Nil;
   avformat_license : Tavformat_license = Nil;
   av_register_all : Tav_register_all = Nil;
   av_register_input_format : Tav_register_input_format = Nil;
   av_register_output_format : Tav_register_output_format = Nil;
   avformat_network_init : Tavformat_network_init = Nil;
   avformat_network_deinit : Tavformat_network_deinit = Nil;
   av_iformat_next : Tav_iformat_next = Nil;
   av_oformat_next : Tav_oformat_next = Nil;
   av_muxer_iterate : Tav_muxer_iterate = Nil;
   av_demuxer_iterate : Tav_demuxer_iterate = Nil;
   avformat_alloc_context : Tavformat_alloc_context = Nil;
   avformat_free_context : Tavformat_free_context = Nil;
   avformat_get_class : Tavformat_get_class = Nil;
   avformat_new_stream : Tavformat_new_stream = Nil;
   av_stream_add_side_data : Tav_stream_add_side_data = Nil;
   av_stream_new_side_data : Tav_stream_new_side_data = Nil;
   av_stream_get_side_data : Tav_stream_get_side_data = Nil;
   av_new_program : Tav_new_program = Nil;
   avformat_alloc_output_context2 : Tavformat_alloc_output_context2 = Nil;
   av_find_input_format : Tav_find_input_format = Nil;
   av_probe_input_format : Tav_probe_input_format = Nil;
   av_probe_input_format2 : Tav_probe_input_format2 = Nil;
   av_probe_input_format3 : Tav_probe_input_format3 = Nil;
   av_probe_input_buffer2 : Tav_probe_input_buffer2 = Nil;
   av_probe_input_buffer : Tav_probe_input_buffer = Nil;
   avformat_open_input : Tavformat_open_input = Nil;
   av_demuxer_open : Tav_demuxer_open = Nil;
   avformat_find_stream_info : Tavformat_find_stream_info = Nil;
   av_find_program_from_stream : Tav_find_program_from_stream = Nil;
   av_program_add_stream_index : Tav_program_add_stream_index = Nil;
   av_find_best_stream : Tav_find_best_stream = Nil;
   av_read_frame : Tav_read_frame = Nil;
   av_seek_frame : Tav_seek_frame = Nil;
   avformat_seek_file : Tavformat_seek_file = Nil;
   avformat_flush : Tavformat_flush = Nil;
   av_read_play : Tav_read_play = Nil;
   av_read_pause : Tav_read_pause = Nil;
   avformat_close_input : Tavformat_close_input = Nil;
   avformat_write_header : Tavformat_write_header = Nil;
   avformat_init_output : Tavformat_init_output = Nil;
   av_write_frame : Tav_write_frame = Nil;
   av_interleaved_write_frame : Tav_interleaved_write_frame = Nil;
   av_write_uncoded_frame : Tav_write_uncoded_frame = Nil;
   av_interleaved_write_uncoded_frame : Tav_interleaved_write_uncoded_frame = Nil;
   av_write_uncoded_frame_query : Tav_write_uncoded_frame_query = Nil;
   av_write_trailer : Tav_write_trailer = Nil;
   av_guess_format : Tav_guess_format = Nil;
   av_guess_codec : Tav_guess_codec = Nil;
   av_get_output_timestamp : Tav_get_output_timestamp = Nil;
   av_hex_dump : Tav_hex_dump = Nil;
   av_hex_dump_log : Tav_hex_dump_log = Nil;
   av_pkt_dump2 : Tav_pkt_dump2 = Nil;
   av_pkt_dump_log2 : Tav_pkt_dump_log2 = Nil;
   av_codec_get_id : Tav_codec_get_id = Nil;
   av_codec_get_tag : Tav_codec_get_tag = Nil;
   av_codec_get_tag2 : Tav_codec_get_tag2 = Nil;
   av_find_default_stream_index : Tav_find_default_stream_index = Nil;
   av_index_search_timestamp : Tav_index_search_timestamp = Nil;
   av_add_index_entry : Tav_add_index_entry = Nil;
   av_url_split : Tav_url_split = Nil;
   av_dump_format : Tav_dump_format = Nil;
   av_get_frame_filename2 : Tav_get_frame_filename2 = Nil;
   av_get_frame_filename : Tav_get_frame_filename = Nil;
   av_filename_number_test : Tav_filename_number_test = Nil;
   av_sdp_create : Tav_sdp_create = Nil;
   av_match_ext : Tav_match_ext = Nil;
   avformat_query_codec : Tavformat_query_codec = Nil;
   avformat_get_riff_video_tags : Tavformat_get_riff_video_tags = Nil;
   avformat_get_riff_audio_tags : Tavformat_get_riff_audio_tags = Nil;
   avformat_get_mov_video_tags : Tavformat_get_mov_video_tags = Nil;
   avformat_get_mov_audio_tags : Tavformat_get_mov_audio_tags = Nil;
   av_guess_sample_aspect_ratio : Tav_guess_sample_aspect_ratio = Nil;
   av_guess_frame_rate : Tav_guess_frame_rate = Nil;
   avformat_match_stream_specifier : Tavformat_match_stream_specifier = Nil;
   avformat_queue_attached_pictures : Tavformat_queue_attached_pictures = Nil;
   av_apply_bitstream_filters : Tav_apply_bitstream_filters = Nil;
   avformat_transfer_internal_stream_timing_info : Tavformat_transfer_internal_stream_timing_info = Nil;
   av_stream_get_codec_timebase : Tav_stream_get_codec_timebase = Nil;   
   av_disposition_from_string : Tav_disposition_from_string = Nil;
   av_disposition_to_string : Tav_disposition_to_string = Nil;  
   avpriv_set_pts_info : Tavpriv_set_pts_info = Nil;   
   avformat_index_get_entries_count : Tavformat_index_get_entries_count = Nil;
   avformat_index_get_entry : Tavformat_index_get_entry = Nil;
   avformat_index_get_entry_from_timestamp : Tavformat_index_get_entry_from_timestamp = Nil;   
   avio_printf : Tavio_printf = Nil;   
   avio_protocol_get_class : Tavio_protocol_get_class = Nil;

//-------------------------------------------------

Function  libavformat_Load(Const aLibName: String = avformatLibName): Boolean;
Procedure libavformat_Free;
Function  libavformat_IsLoaded: boolean;

//==============================================================
//==============================================================
implementation

Const
   InvalidLibHandle = 0;

var
  avformat_Handle: TLibHandle;

function libavformat_IsLoaded: boolean;
begin
  Result := (avformat_Handle <> InvalidLibHandle);
end;

Procedure libavformat_Free;
begin
  if libavformat_IsLoaded then
  begin
    DynLibs.UnloadLibrary(avformat_Handle);
    avformat_Handle := InvalidLibHandle;
  end;
end;

function libavformat_Load(Const aLibName: String = avformatLibName): Boolean;
//.............................
   Function _GetProcAddress(Const aProcName: String): {$IFDEF cpui8086}FarPointer{$ELSE}Pointer{$ENDIF};
   Begin
      result := Nil;
      If aProcName = '' Then
         exit;

      result := dynlibs.GetProcedureAddress(avformat_Handle, AnsiString(aProcName));
   End;
   //.....................................
var
  thelib: string;

{$IF DEFINED(UNIX) and (NOT DEFINED(DARWIN))}
  ssl:Integer;
  ssp:Integer;
{$ENDIF}
begin
  Result := False;

  if avformat_Handle <> InvalidLibHandle then Exit(True);

  if aLibName = '' then
     thelib := avformatLibName else
     thelib := aLibName;

  avformat_Handle := dynlibs.LoadLibrary(thelib);

   //.....................................
   // On UNIX If Library NOT load, try to load .so file
  {$IF DEFINED(UNIX) and (NOT DEFINED(DARWIN))}
   If avformat_Handle = 0 Then
   Begin
      ssl := length(thelib);
      ssp := Pos('.so.', thelib);
      If ssp > 0 Then
         delete(thelib, ssp, ssl - ssp + 1);

      avformat_Handle := dynlibs.LoadLibrary(thelib + '.so');
   End;
  {$ENDIF}

    if avformat_Handle = InvalidLibHandle then Exit(False);

    //.........................................

    Pointer(avio_find_protocol_name) := _GetProcAddress('avio_find_protocol_name');
    Pointer(avio_check) := _GetProcAddress('avio_check');
    Pointer(avpriv_io_move) := _GetProcAddress('avpriv_io_move');
    Pointer(avpriv_io_delete) := _GetProcAddress('avpriv_io_delete');
    Pointer(avio_open_dir) := _GetProcAddress('avio_open_dir');
    Pointer(avio_read_dir) := _GetProcAddress('avio_read_dir');
    Pointer(avio_close_dir) := _GetProcAddress('avio_close_dir');
    Pointer(avio_free_directory_entry) := _GetProcAddress('avio_free_directory_entry');
    Pointer(avio_alloc_context) := _GetProcAddress('avio_alloc_context');
    Pointer(avio_context_free) := _GetProcAddress('avio_context_free');
    Pointer(avio_w8) := _GetProcAddress('avio_w8');
    Pointer(avio_write) := _GetProcAddress('avio_write');
    Pointer(avio_wl64) := _GetProcAddress('avio_wl64');
    Pointer(avio_wb64) := _GetProcAddress('avio_wb64');
    Pointer(avio_wl32) := _GetProcAddress('avio_wl32');
    Pointer(avio_wb32) := _GetProcAddress('avio_wb32');
    Pointer(avio_wl24) := _GetProcAddress('avio_wl24');
    Pointer(avio_wb24) := _GetProcAddress('avio_wb24');
    Pointer(avio_wl16) := _GetProcAddress('avio_wl16');
    Pointer(avio_wb16) := _GetProcAddress('avio_wb16');
    Pointer(avio_put_str) := _GetProcAddress('avio_put_str');
    Pointer(avio_put_str16le) := _GetProcAddress('avio_put_str16le');
    Pointer(avio_put_str16be) := _GetProcAddress('avio_put_str16be');
    Pointer(avio_write_marker) := _GetProcAddress('avio_write_marker');
    Pointer(avio_seek) := _GetProcAddress('avio_seek');
    Pointer(avio_skip) := _GetProcAddress('avio_skip');
    Pointer(avio_size) := _GetProcAddress('avio_size');
    Pointer(avio_feof) := _GetProcAddress('avio_feof');
    Pointer(avio_flush) := _GetProcAddress('avio_flush');
    Pointer(avio_read) := _GetProcAddress('avio_read');
    Pointer(avio_read_partial) := _GetProcAddress('avio_read_partial');
    Pointer(avio_r8) := _GetProcAddress('avio_r8');
    Pointer(avio_rl16) := _GetProcAddress('avio_rl16');
    Pointer(avio_rl24) := _GetProcAddress('avio_rl24');
    Pointer(avio_rl32) := _GetProcAddress('avio_rl32');
    Pointer(avio_rl64) := _GetProcAddress('avio_rl64');
    Pointer(avio_rb16) := _GetProcAddress('avio_rb16');
    Pointer(avio_rb24) := _GetProcAddress('avio_rb24');
    Pointer(avio_rb32) := _GetProcAddress('avio_rb32');
    Pointer(avio_rb64) := _GetProcAddress('avio_rb64');
    Pointer(avio_get_str) := _GetProcAddress('avio_get_str');
    Pointer(avio_get_str16le) := _GetProcAddress('avio_get_str16le');
    Pointer(avio_get_str16be) := _GetProcAddress('avio_get_str16be');
    Pointer(avio_open) := _GetProcAddress('avio_open');
    Pointer(avio_open2) := _GetProcAddress('avio_open2');
    Pointer(avio_close) := _GetProcAddress('avio_close');
    Pointer(avio_closep) := _GetProcAddress('avio_closep');
    Pointer(avio_open_dyn_buf) := _GetProcAddress('avio_open_dyn_buf');
    Pointer(avio_get_dyn_buf) := _GetProcAddress('avio_get_dyn_buf');
    Pointer(avio_close_dyn_buf) := _GetProcAddress('avio_close_dyn_buf');
    Pointer(avio_enum_protocols) := _GetProcAddress('avio_enum_protocols');
    Pointer(avio_pause) := _GetProcAddress('avio_pause');
    Pointer(avio_seek_time) := _GetProcAddress('avio_seek_time');
    Pointer(avio_read_to_bprint) := _GetProcAddress('avio_read_to_bprint');
    Pointer(avio_accept) := _GetProcAddress('avio_accept');
    Pointer(avio_handshake) := _GetProcAddress('avio_handshake');
    Pointer(av_get_packet) := _GetProcAddress('av_get_packet');
    Pointer(av_append_packet) := _GetProcAddress('av_append_packet');
    Pointer(av_stream_get_r_frame_rate) := _GetProcAddress('av_stream_get_r_frame_rate');
    Pointer(av_stream_set_r_frame_rate) := _GetProcAddress('av_stream_set_r_frame_rate');
    Pointer(av_stream_get_recommended_encoder_configuration) := _GetProcAddress('av_stream_get_recommended_encoder_configuration');
    Pointer(av_stream_set_recommended_encoder_configuration) := _GetProcAddress('av_stream_set_recommended_encoder_configuration');
    Pointer(av_stream_get_parser) := _GetProcAddress('av_stream_get_parser');
    Pointer(av_stream_get_end_pts) := _GetProcAddress('av_stream_get_end_pts');
    Pointer(av_format_get_probe_score) := _GetProcAddress('av_format_get_probe_score');
    Pointer(av_format_get_video_codec) := _GetProcAddress('av_format_get_video_codec');
    Pointer(av_format_set_video_codec) := _GetProcAddress('av_format_set_video_codec');
    Pointer(av_format_get_audio_codec) := _GetProcAddress('av_format_get_audio_codec');
    Pointer(av_format_set_audio_codec) := _GetProcAddress('av_format_set_audio_codec');
    Pointer(av_format_get_subtitle_codec) := _GetProcAddress('av_format_get_subtitle_codec');
    Pointer(av_format_set_subtitle_codec) := _GetProcAddress('av_format_set_subtitle_codec');
    Pointer(av_format_get_data_codec) := _GetProcAddress('av_format_get_data_codec');
    Pointer(av_format_set_data_codec) := _GetProcAddress('av_format_set_data_codec');
    Pointer(av_format_get_metadata_header_padding) := _GetProcAddress('av_format_get_metadata_header_padding');
    Pointer(av_format_set_metadata_header_padding) := _GetProcAddress('av_format_set_metadata_header_padding');
    Pointer(av_format_get_opaque) := _GetProcAddress('av_format_get_opaque');
    Pointer(av_format_set_opaque) := _GetProcAddress('av_format_set_opaque');
    Pointer(av_format_get_control_message_cb) := _GetProcAddress('av_format_get_control_message_cb');
    Pointer(av_format_set_control_message_cb) := _GetProcAddress('av_format_set_control_message_cb');
    Pointer(av_format_get_open_cb) := _GetProcAddress('av_format_get_open_cb');
    Pointer(av_format_set_open_cb) := _GetProcAddress('av_format_set_open_cb');
    Pointer(av_format_inject_global_side_data) := _GetProcAddress('av_format_inject_global_side_data');
    Pointer(av_fmt_ctx_get_duration_estimation_method) := _GetProcAddress('av_fmt_ctx_get_duration_estimation_method');
    Pointer(avformat_version) := _GetProcAddress('avformat_version');
    Pointer(avformat_configuration) := _GetProcAddress('avformat_configuration');
    Pointer(avformat_license) := _GetProcAddress('avformat_license');
    Pointer(av_register_all) := _GetProcAddress('av_register_all');
    Pointer(av_register_input_format) := _GetProcAddress('av_register_input_format');
    Pointer(av_register_output_format) := _GetProcAddress('av_register_output_format');
    Pointer(avformat_network_init) := _GetProcAddress('avformat_network_init');
    Pointer(avformat_network_deinit) := _GetProcAddress('avformat_network_deinit');
    Pointer(av_iformat_next) := _GetProcAddress('av_iformat_next');
    Pointer(av_oformat_next) := _GetProcAddress('av_oformat_next');
    Pointer(av_muxer_iterate) := _GetProcAddress('av_muxer_iterate');
    Pointer(av_demuxer_iterate) := _GetProcAddress('av_demuxer_iterate');
    Pointer(avformat_alloc_context) := _GetProcAddress('avformat_alloc_context');
    Pointer(avformat_free_context) := _GetProcAddress('avformat_free_context');
    Pointer(avformat_get_class) := _GetProcAddress('avformat_get_class');
    Pointer(avformat_new_stream) := _GetProcAddress('avformat_new_stream');
    Pointer(av_stream_add_side_data) := _GetProcAddress('av_stream_add_side_data');
    Pointer(av_stream_new_side_data) := _GetProcAddress('av_stream_new_side_data');
    Pointer(av_stream_get_side_data) := _GetProcAddress('av_stream_get_side_data');
    Pointer(av_new_program) := _GetProcAddress('av_new_program');
    Pointer(avformat_alloc_output_context2) := _GetProcAddress('avformat_alloc_output_context2');
    Pointer(av_find_input_format) := _GetProcAddress('av_find_input_format');
    Pointer(av_probe_input_format) := _GetProcAddress('av_probe_input_format');
    Pointer(av_probe_input_format2) := _GetProcAddress('av_probe_input_format2');
    Pointer(av_probe_input_format3) := _GetProcAddress('av_probe_input_format3');
    Pointer(av_probe_input_buffer2) := _GetProcAddress('av_probe_input_buffer2');
    Pointer(av_probe_input_buffer) := _GetProcAddress('av_probe_input_buffer');
    Pointer(avformat_open_input) := _GetProcAddress('avformat_open_input');
    Pointer(av_demuxer_open) := _GetProcAddress('av_demuxer_open');
    Pointer(avformat_find_stream_info) := _GetProcAddress('avformat_find_stream_info');
    Pointer(av_find_program_from_stream) := _GetProcAddress('av_find_program_from_stream');
    Pointer(av_program_add_stream_index) := _GetProcAddress('av_program_add_stream_index');
    Pointer(av_find_best_stream) := _GetProcAddress('av_find_best_stream');
    Pointer(av_read_frame) := _GetProcAddress('av_read_frame');
    Pointer(av_seek_frame) := _GetProcAddress('av_seek_frame');
    Pointer(avformat_seek_file) := _GetProcAddress('avformat_seek_file');
    Pointer(avformat_flush) := _GetProcAddress('avformat_flush');
    Pointer(av_read_play) := _GetProcAddress('av_read_play');
    Pointer(av_read_pause) := _GetProcAddress('av_read_pause');
    Pointer(avformat_close_input) := _GetProcAddress('avformat_close_input');
    Pointer(avformat_write_header) := _GetProcAddress('avformat_write_header');
    Pointer(avformat_init_output) := _GetProcAddress('avformat_init_output');
    Pointer(av_write_frame) := _GetProcAddress('av_write_frame');
    Pointer(av_interleaved_write_frame) := _GetProcAddress('av_interleaved_write_frame');
    Pointer(av_write_uncoded_frame) := _GetProcAddress('av_write_uncoded_frame');
    Pointer(av_interleaved_write_uncoded_frame) := _GetProcAddress('av_interleaved_write_uncoded_frame');
    Pointer(av_write_uncoded_frame_query) := _GetProcAddress('av_write_uncoded_frame_query');
    Pointer(av_write_trailer) := _GetProcAddress('av_write_trailer');
    Pointer(av_guess_format) := _GetProcAddress('av_guess_format');
    Pointer(av_guess_codec) := _GetProcAddress('av_guess_codec');
    Pointer(av_get_output_timestamp) := _GetProcAddress('av_get_output_timestamp');
    Pointer(av_hex_dump) := _GetProcAddress('av_hex_dump');
    Pointer(av_hex_dump_log) := _GetProcAddress('av_hex_dump_log');
    Pointer(av_pkt_dump2) := _GetProcAddress('av_pkt_dump2');
    Pointer(av_pkt_dump_log2) := _GetProcAddress('av_pkt_dump_log2');
    Pointer(av_codec_get_id) := _GetProcAddress('av_codec_get_id');
    Pointer(av_codec_get_tag) := _GetProcAddress('av_codec_get_tag');
    Pointer(av_codec_get_tag2) := _GetProcAddress('av_codec_get_tag2');
    Pointer(av_find_default_stream_index) := _GetProcAddress('av_find_default_stream_index');
    Pointer(av_index_search_timestamp) := _GetProcAddress('av_index_search_timestamp');
    Pointer(av_add_index_entry) := _GetProcAddress('av_add_index_entry');
    Pointer(av_url_split) := _GetProcAddress('av_url_split');
    Pointer(av_dump_format) := _GetProcAddress('av_dump_format');
    Pointer(av_get_frame_filename2) := _GetProcAddress('av_get_frame_filename2');
    Pointer(av_get_frame_filename) := _GetProcAddress('av_get_frame_filename');
    Pointer(av_filename_number_test) := _GetProcAddress('av_filename_number_test');
    Pointer(av_sdp_create) := _GetProcAddress('av_sdp_create');
    Pointer(av_match_ext) := _GetProcAddress('av_match_ext');
    Pointer(avformat_query_codec) := _GetProcAddress('avformat_query_codec');
    Pointer(avformat_get_riff_video_tags) := _GetProcAddress('avformat_get_riff_video_tags');
    Pointer(avformat_get_riff_audio_tags) := _GetProcAddress('avformat_get_riff_audio_tags');
    Pointer(avformat_get_mov_video_tags) := _GetProcAddress('avformat_get_mov_video_tags');
    Pointer(avformat_get_mov_audio_tags) := _GetProcAddress('avformat_get_mov_audio_tags');
    Pointer(av_guess_sample_aspect_ratio) := _GetProcAddress('av_guess_sample_aspect_ratio');
    Pointer(av_guess_frame_rate) := _GetProcAddress('av_guess_frame_rate');
    Pointer(avformat_match_stream_specifier) := _GetProcAddress('avformat_match_stream_specifier');
    Pointer(avformat_queue_attached_pictures) := _GetProcAddress('avformat_queue_attached_pictures');
    Pointer(av_apply_bitstream_filters) := _GetProcAddress('av_apply_bitstream_filters');
    Pointer(avformat_transfer_internal_stream_timing_info) := _GetProcAddress('avformat_transfer_internal_stream_timing_info');
    Pointer(av_stream_get_codec_timebase) := _GetProcAddress('av_stream_get_codec_timebase'); 
    Pointer(av_disposition_from_string) := _GetProcAddress('av_disposition_from_string');
    Pointer(av_disposition_to_string) := _GetProcAddress('av_disposition_to_string');   
    Pointer(avpriv_set_pts_info) := _GetProcAddress('avpriv_set_pts_info'); 
    Pointer(avformat_index_get_entries_count) := _GetProcAddress('avformat_index_get_entries_count');
    Pointer(avformat_index_get_entry) := _GetProcAddress('avformat_index_get_entry');
    Pointer(avformat_index_get_entry_from_timestamp) := _GetProcAddress('avformat_index_get_entry_from_timestamp'); 
    Pointer(avio_printf) := _GetProcAddress('avio_printf'); 
    Pointer(avio_protocol_get_class) := _GetProcAddress('avio_protocol_get_class');


    //.........................................

    Result := True;

end;

end.
