{**********************************************************************
                Copyright (c) PilotLogic Software House

 Package pl_FFmpeg5
 This file is part of CodeTyphon Studio (https://www.pilotlogic.com/)

 ***** BEGIN LICENSE BLOCK *****
 * Version: LGPL-2.1 with link exception (Modified LGPL)
 *
 * The contents of this file are subject to the
 * GNU LIBRARY GENERAL PUBLIC LICENSE Version 2.1 (the "License")
 * with the following modification:
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent modules,
 * and to copy and distribute the resulting executable under terms of your choice,
 * provided that you also meet, for each linked independent module, the terms
 * and conditions of the license of that module. An independent module is a
 * module which is not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library, but
 * you are not obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * ***** END LICENSE BLOCK *****

***********************************************************************}

Unit libpostproc;

{$MODE objfpc}{$H+}

Interface

Uses          
  ctypes,
  Classes, dynlibs,
  ffmpeg_types;

Const
{$IF defined(MSWINDOWS)}
   postprocLibName = 'postproc-56.dll';
{$ELSEIF defined(LINUX)}
   postprocLibName = 'libpostproc.so.56';
{$ELSEIF defined(DARWIN)}
   postprocLibName = 'libpostproc.56.dylib';
{$ELSE}
   postprocLibName = 'libpostproc.so.56';
{$ENDIF}

Const
   PP_QUALITY_MAX = 6;

   PP_CPU_CAPS_MMX = $80000000;
   PP_CPU_CAPS_MMX2 = $20000000;
   PP_CPU_CAPS_3DNOW = $40000000;
   PP_CPU_CAPS_ALTIVEC = $10000000;
   PP_CPU_CAPS_AUTO = $00080000;

   PP_FORMAT = $00000008;
   PP_FORMAT_420 = ($00000011 Or PP_FORMAT);
   PP_FORMAT_422 = ($00000001 Or PP_FORMAT);
   PP_FORMAT_411 = ($00000002 Or PP_FORMAT);
   PP_FORMAT_444 = ($00000000 Or PP_FORMAT);
   PP_FORMAT_440 = ($00000010 Or PP_FORMAT);

   PP_PICT_TYPE_QP2 = $00000010;

Type
   ppp_context = ^pp_context;
   pp_context = Record
   End;

   ppp_mode = ^pp_mode;
   pp_mode = Record
   End;

   Tpp_src_puint8_t = Array[0..2] Of puint8_t;
   Tpp_dst_puint8_t = Tpp_src_puint8_t;
   Tpp_srcStride_int = Array[0..2] Of int;
   Tpp_dstStride_int = Tpp_srcStride_int;

  //...................................................

Type

   Tpostproc_version = function(): unsigned; cdecl;
   Tpostproc_configuration = function(): pAnsiChar; cdecl;
   Tpostproc_license = function(): pAnsiChar; cdecl;
   Tpp_postprocess = Procedure(Const src: Tpp_src_puint8_t; Const srcStride: Tpp_srcStride_int; dst: Tpp_dst_puint8_t;
                               Const dstStride: Tpp_dstStride_int; horizontalSize: int; verticalSize: int;
                               Const QP_store: pint8_t; QP_stride: int; mode: ppp_mode;
                               ppContext: ppp_context; pict_type: int); cdecl;
   Tpp_get_mode_by_name_and_quality = function(Const name: pAnsiChar; quality: int): ppp_mode; cdecl;
   Tpp_free_mode = Procedure(mode: ppp_mode); cdecl;
   Tpp_get_context = function(width: int; height: int; flags: int): ppp_context; cdecl;
   Tpp_free_context = Procedure(ppContext: ppp_context); cdecl;

  //....................................................

Var

   postproc_version : Tpostproc_version = Nil;
   postproc_configuration : Tpostproc_configuration = Nil;
   postproc_license : Tpostproc_license  = Nil;
   pp_postprocess : Tpp_postprocess = Nil;
   pp_get_mode_by_name_and_quality : Tpp_get_mode_by_name_and_quality = Nil;
   pp_free_mode : Tpp_free_mode = Nil;
   pp_get_context : Tpp_get_context = Nil;
   pp_free_context : Tpp_free_context = Nil;

//.........................

Function  libpostproc_Load(Const aLibName: String = postprocLibName): Boolean;
Procedure libpostproc_Free;
Function  libpostproc_IsLoaded: boolean;

//==============================================================
//==============================================================
implementation

Const
   InvalidLibHandle = 0;

var
  postproc_Handle: TLibHandle;

function libpostproc_IsLoaded: boolean;
begin
  Result := (postproc_Handle <> InvalidLibHandle);
end;

Procedure libpostproc_Free;
begin
  if libpostproc_IsLoaded then
  begin
    DynLibs.UnloadLibrary(postproc_Handle);
    postproc_Handle := InvalidLibHandle;
  end;
end;

function libpostproc_Load(Const aLibName: String = postprocLibName): Boolean;
//.............................
   Function _GetProcAddress(Const aProcName: String): {$IFDEF cpui8086}FarPointer{$ELSE}Pointer{$ENDIF};
   Begin
      result := Nil;
      If aProcName = '' Then
         exit;

      result := dynlibs.GetProcedureAddress(postproc_Handle, AnsiString(aProcName));
   End;
   //.....................................
var
  thelib: string;

{$IF DEFINED(UNIX) and (NOT DEFINED(DARWIN))}
  ssl:Integer;
  ssp:Integer;
{$ENDIF}
begin
  Result := False;

  if postproc_Handle <> InvalidLibHandle then Exit(True);

  if aLibName = '' then
     thelib := postprocLibName else
     thelib := aLibName;

  postproc_Handle := dynlibs.LoadLibrary(thelib);

   //.....................................
   // On UNIX If Library NOT load, try to load .so file
  {$IF DEFINED(UNIX) and (NOT DEFINED(DARWIN))}
   If postproc_Handle = 0 Then
   Begin
      ssl := length(thelib);
      ssp := Pos('.so.', thelib);
      If ssp > 0 Then
         delete(thelib, ssp, ssl - ssp + 1);

      postproc_Handle := dynlibs.LoadLibrary(thelib + '.so');
   End;
  {$ENDIF}

    if postproc_Handle = InvalidLibHandle then Exit(False);

    //.........................................

    Pointer(postproc_version) := _GetProcAddress('postproc_version');
    Pointer(postproc_configuration) := _GetProcAddress('postproc_configuration');
    Pointer(postproc_license) := _GetProcAddress('postproc_license');
    Pointer(pp_postprocess) := _GetProcAddress('pp_postprocess');
    Pointer(pp_get_mode_by_name_and_quality) := _GetProcAddress('pp_get_mode_by_name_and_quality');
    Pointer(pp_free_mode) := _GetProcAddress('pp_free_mode');
    Pointer(pp_get_context) := _GetProcAddress('pp_get_context');
    Pointer(pp_free_context) := _GetProcAddress('pp_free_context');

    //.........................................

    Result := True;

end;

end.
