﻿{**********************************************************************
                Copyright (c) PilotLogic Software House

 Package pl_FFmpeg5
 This file is part of CodeTyphon Studio (https://www.pilotlogic.com/)

 ***** BEGIN LICENSE BLOCK *****
 * Version: LGPL-2.1 with link exception (Modified LGPL)
 *
 * The contents of this file are subject to the
 * GNU LIBRARY GENERAL PUBLIC LICENSE Version 2.1 (the "License")
 * with the following modification:
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent modules,
 * and to copy and distribute the resulting executable under terms of your choice,
 * provided that you also meet, for each linked independent module, the terms
 * and conditions of the license of that module. An independent module is a
 * module which is not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library, but
 * you are not obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * ***** END LICENSE BLOCK *****

***********************************************************************}

unit libswscale;

{$MODE objfpc}{$H+}

Interface

Uses
  ctypes,
  Classes, dynlibs,
  ffmpeg_types, libavutil;

Const
{$IF defined(MSWINDOWS)}
   swscaleLibName = 'swscale-6.dll';
{$ELSEIF defined(LINUX)}
   swscaleLibName = 'libswscale.so.6';
{$ELSEIF defined(DARWIN)}
   swscaleLibName = 'libswscale.6.dylib';
{$ELSE}
   swscaleLibName = 'libswscale.so.6';
{$ENDIF}

Const
   SWS_FAST_BILINEAR = 1;
   SWS_BILINEAR = 2;
   SWS_BICUBIC = 4;
   SWS_X = 8;
   SWS_POINT = $10;
   SWS_AREA = $20;
   SWS_BICUBLIN = $40;
   SWS_GAUSS = $80;
   SWS_SINC = $100;
   SWS_LANCZOS = $200;
   SWS_SPLINE = $400;
   SWS_SRC_V_CHR_DROP_MASK = $30000;
   SWS_SRC_V_CHR_DROP_SHIFT = 16;
   SWS_PARAM_DEFAULT = 123456;
   SWS_PRINT_INFO = $1000;
   SWS_FULL_CHR_H_INT = $2000;
   SWS_FULL_CHR_H_INP = $4000;
   SWS_DIRECT_BGR = $8000;
   SWS_ACCURATE_RND = $40000;
   SWS_BITEXACT = $80000;
   SWS_ERROR_DIFFUSION = $800000;
   SWS_MAX_REDUCE_CUTOFF = 0.002;
   SWS_CS_ITU709 = 1;
   SWS_CS_FCC = 4;
   SWS_CS_ITU601 = 5;
   SWS_CS_ITU624 = 5;
   SWS_CS_SMPTE170M = 5;
   SWS_CS_SMPTE240M = 7;
   SWS_CS_DEFAULT = 5;
   SWS_CS_BT2020 = 9;

Type
   SwsVector = Record
      coeff: pdouble;
      length: int;
   End;

   pSwsVector = ^SwsVector;
   SwsFilter = Record
      lumH: pSwsVector;
      lumV: pSwsVector;
      chrH: pSwsVector;
      chrV: pSwsVector;
   End;

   pSwsFilter = ^SwsFilter;
   SwsContext = Record
   End;

   pSwsContext = ^SwsContext;
   Tsws_array_uint8_t = array_uint8_t;
   psws_array_uint8_t = ^Tsws_array_uint8_t;
   Tsws_array_int = array_int;
   psws_array_int = ^Tsws_array_int;
   Tsws_array4_int = array4_int;
   psws_array4_int = ^Tsws_array4_int;

  //...................................................

Type

   Tswscale_version = function(): unsigned; cdecl;
   Tswscale_configuration = function(): pAnsiChar; cdecl;
   Tswscale_license = function(): pAnsiChar; cdecl;
   Tsws_getCoefficients = function(colorspace: int): pInt; cdecl;
   Tsws_isSupportedInput = function(pix_fmt: AVPixelFormat): int; cdecl;
   Tsws_isSupportedOutput = function(pix_fmt: AVPixelFormat): int; cdecl;
   Tsws_isSupportedEndiannessConversion = function(pix_fmt: AVPixelFormat): int; cdecl;
   Tsws_alloc_context = function(): pSwsContext; cdecl;
   Tsws_init_context = function(sws_context: pSwsContext; srcFilter: pSwsFilter; dstFilter: pSwsFilter): int; cdecl;
   Tsws_freeContext = Procedure(SwsContext: pSwsContext); cdecl;
   Tsws_getContext = function(srcW: int; srcH: int; srcFormat: AVPixelFormat; dstW: int; dstH: int; dstFormat: AVPixelFormat; flags: int;
                        srcFilter: pSwsFilter; dstFilter: pSwsFilter; Const param: pdouble): pSwsContext; cdecl;
   Tsws_scale = function(c: pSwsContext; Const srcSlice: psws_array_uint8_t; Const srcStride: psws_array_int; srcSliceY: int; srcSliceH: int;
                   dst: psws_array_uint8_t;
                    Const dstStride: psws_array_int): int; cdecl;
   Tsws_setColorspaceDetails = function(c: pSwsContext; Const inv_table: psws_array4_int; srcRange: int; Const table: psws_array4_int;
                                 dstRange: int; brightness: int; contrast: int; saturation: int): int; cdecl;
   Tsws_getColorspaceDetails = function(c: pSwsContext; Var inv_table: pInt; Var srcRange: int; Var table: pInt; Var dstRange: int;
                                  Var brightness: int; Var contrast: int; Var saturation: int): int; cdecl;
   Tsws_allocVec = function(length: int): pSwsVector; cdecl;
   Tsws_getGaussianVec = function(variance: double; quality: double): pSwsVector; cdecl;
   Tsws_scaleVec = Procedure(a: pSwsVector; scalar: double); cdecl;
   Tsws_normalizeVec = Procedure(a: pSwsVector; height: double); cdecl;
   Tsws_getConstVec = function(c: double; length: int): pSwsVector; cdecl;
   Tsws_getIdentityVec = function(): pSwsVector; cdecl;
   Tsws_convVec = Procedure(a: pSwsVector; b: pSwsVector); cdecl;
   Tsws_addVec = Procedure(a: pSwsVector; b: pSwsVector); cdecl;
   Tsws_subVec = Procedure(a: pSwsVector; b: pSwsVector); cdecl;
   Tsws_shiftVec = Procedure(a: pSwsVector; shift: int); cdecl;
   Tsws_cloneVec = function(a: pSwsVector): pSwsVector; cdecl;
   Tsws_printVec2 = Procedure(a: pSwsVector; log_ctx: pAVClass; log_level: int); cdecl;
   Tsws_freeVec = Procedure(a: pSwsVector); cdecl;
   Tsws_getDefaultFilter = function(lumaGBlur: float; chromaGBlur: float; lumaSharpen: float; chromaSharpen: float; chromaHShift:
                              float; chromaVShift: float;verbose: int): pSwsFilter; cdecl;
   Tsws_freeFilter = Procedure(filter: pSwsFilter); cdecl;
   Tsws_getCachedContext = function(context: pSwsContext; srcW: int; srcH: int; srcFormat: AVPixelFormat; dstW: int; dstH: int;
                              dstFormat: AVPixelFormat; flags: int; srcFilter: pSwsFilter; dstFilter: pSwsFilter;
                              Const param: pdouble): pSwsContext; cdecl;
   Tsws_convertPalette8ToPacked32 = Procedure(Const src: puint8_t; Var dst: uint8_t; num_pixels: int; Const palette: puint8_t); cdecl;
   Tsws_convertPalette8ToPacked24 = Procedure(Const src: puint8_t; Var dst: uint8_t; num_pixels: int; Const palette: puint8_t); cdecl;
   Tsws_get_class = function(): pAVClass; cdecl;
   Tsws_scale_frame = Function(c: PSwsContext; dst: PAVFrame; Const src: PAVFrame): Integer; cdecl;
   Tsws_frame_start = Function(c: PSwsContext; dst: PAVFrame; Const src: PAVFrame): Integer; cdecl;
   Tsws_frame_end = Procedure(c: PSwsContext); cdecl;
   Tsws_send_slice = Function(c: PSwsContext; slice_start, slice_height: Cardinal): Integer; cdecl;
   Tsws_receive_slice = Function(c: PSwsContext; slice_start, slice_height: Cardinal): Integer; cdecl;
   Tsws_receive_slice_alignment = Function(Const c: PSwsContext): Cardinal; cdecl;


  //....................................................

Var

   swscale_version : Tswscale_version = Nil;
   swscale_configuration : Tswscale_configuration = Nil;
   swscale_license : Tswscale_license = Nil;
   sws_getCoefficients : Tsws_getCoefficients = Nil;
   sws_isSupportedInput : Tsws_isSupportedInput = Nil;
   sws_isSupportedOutput : Tsws_isSupportedOutput = Nil;
   sws_isSupportedEndiannessConversion : Tsws_isSupportedEndiannessConversion = Nil;
   sws_alloc_context : Tsws_alloc_context = Nil;
   sws_init_context : Tsws_init_context = Nil;
   sws_freeContext : Tsws_freeContext = Nil;
   sws_getContext : Tsws_getContext = Nil;
   sws_scale : Tsws_scale = Nil;
   sws_setColorspaceDetails : Tsws_setColorspaceDetails = Nil;
   sws_getColorspaceDetails : Tsws_getColorspaceDetails = Nil;
   sws_allocVec : Tsws_allocVec = Nil;
   sws_getGaussianVec : Tsws_getGaussianVec = Nil;
   sws_scaleVec : Tsws_scaleVec = Nil;
   sws_normalizeVec : Tsws_normalizeVec = Nil;
   sws_getConstVec : Tsws_getConstVec = Nil;
   sws_getIdentityVec : Tsws_getIdentityVec = Nil;
   sws_convVec : Tsws_convVec = Nil;
   sws_addVec : Tsws_addVec = Nil;
   sws_subVec : Tsws_subVec = Nil;
   sws_shiftVec : Tsws_shiftVec = Nil;
   sws_cloneVe : Tsws_cloneVec = Nil;
   sws_printVec2 : Tsws_printVec2 = Nil;
   sws_freeVec : Tsws_freeVec = Nil;
   sws_getDefaultFilter : Tsws_getDefaultFilter = Nil;
   sws_freeFilter : Tsws_freeFilter = Nil;
   sws_getCachedContext : Tsws_getCachedContext = Nil;
   sws_convertPalette8ToPacked32 : Tsws_convertPalette8ToPacked32 = Nil;
   sws_convertPalette8ToPacked24 : Tsws_convertPalette8ToPacked24 = Nil;
   sws_get_class : Tsws_get_class = Nil;      
   sws_scale_frame : Tsws_scale_frame = Nil;
   sws_frame_start : Tsws_frame_start = Nil;
   sws_frame_end : Tsws_frame_end = Nil;
   sws_send_slice : Tsws_send_slice = Nil;
   sws_receive_slice : Tsws_receive_slice = Nil;
   sws_receive_slice_alignment : Tsws_receive_slice_alignment = Nil;

//.........................

Function  libswscale_Load(Const aLibName: String = swscaleLibName): Boolean;
Procedure libswscale_Free;
Function  libswscale_IsLoaded: boolean;

//==============================================================
//==============================================================
implementation

Const
   InvalidLibHandle = 0;

var
  swscale_Handle: TLibHandle;

function libswscale_IsLoaded: boolean;
begin
  Result := (swscale_Handle <> InvalidLibHandle);
end;

Procedure libswscale_Free;
begin
  if libswscale_IsLoaded then
  begin
    DynLibs.UnloadLibrary(swscale_Handle);
    swscale_Handle := InvalidLibHandle;
  end;
end;

function libswscale_Load(Const aLibName: String = swscaleLibName): Boolean;
//.............................
   Function _GetProcAddress(Const aProcName: String): {$IFDEF cpui8086}FarPointer{$ELSE}Pointer{$ENDIF};
   Begin
      result := Nil;
      If aProcName = '' Then
         exit;

      result := dynlibs.GetProcedureAddress(swscale_Handle, AnsiString(aProcName));
   End;
   //.....................................
var
  thelib: string;

{$IF DEFINED(UNIX) and (NOT DEFINED(DARWIN))}
  ssl:Integer;
  ssp:Integer;
{$ENDIF}
begin
  Result := False;

  if swscale_Handle <> InvalidLibHandle then Exit(True);

  if aLibName = '' then
     thelib := swscaleLibName else
     thelib := aLibName;

  swscale_Handle := dynlibs.LoadLibrary(thelib);

   //.....................................
   // On UNIX If Library NOT load, try to load .so file
  {$IF DEFINED(UNIX) and (NOT DEFINED(DARWIN))}
   If swscale_Handle = 0 Then
   Begin
      ssl := length(thelib);
      ssp := Pos('.so.', thelib);
      If ssp > 0 Then
         delete(thelib, ssp, ssl - ssp + 1);

      swscale_Handle := dynlibs.LoadLibrary(thelib + '.so');
   End;
  {$ENDIF}

    if swscale_Handle = InvalidLibHandle then Exit(False);

    //.........................................

    Pointer(swscale_version) := _GetProcAddress('swscale_version');
    Pointer(swscale_configuration) := _GetProcAddress('swscale_configuration');
    Pointer(swscale_license) := _GetProcAddress('swscale_license');
    Pointer(sws_getCoefficients) := _GetProcAddress('sws_getCoefficients');
    Pointer(sws_isSupportedInput) := _GetProcAddress('sws_isSupportedInput');
    Pointer(sws_isSupportedOutput) := _GetProcAddress('sws_isSupportedOutput');
    Pointer(sws_isSupportedEndiannessConversion) := _GetProcAddress('sws_isSupportedEndiannessConversion');
    Pointer(sws_alloc_context) := _GetProcAddress('sws_alloc_context');
    Pointer(sws_init_context) := _GetProcAddress('sws_init_context');
    Pointer(sws_freeContext) := _GetProcAddress('sws_freeContext');
    Pointer(sws_getContext) := _GetProcAddress('sws_getContext');
    Pointer(sws_scale) := _GetProcAddress('sws_scale');
    Pointer(sws_setColorspaceDetails) := _GetProcAddress('sws_setColorspaceDetails');
    Pointer(sws_getColorspaceDetails) := _GetProcAddress('sws_getColorspaceDetails');
    Pointer(sws_allocVec) := _GetProcAddress('sws_allocVec');
    Pointer(sws_getGaussianVec) := _GetProcAddress('sws_getGaussianVec');
    Pointer(sws_scaleVec) := _GetProcAddress('sws_scaleVec');
    Pointer(sws_normalizeVec) := _GetProcAddress('sws_normalizeVec');
    Pointer(sws_getConstVec) := _GetProcAddress('sws_getConstVec');
    Pointer(sws_getIdentityVec) := _GetProcAddress('sws_getIdentityVec');
    Pointer(sws_convVec) := _GetProcAddress('sws_convVec');
    Pointer(sws_addVec) := _GetProcAddress('sws_addVec');
    Pointer(sws_subVec) := _GetProcAddress('sws_subVec');
    Pointer(sws_shiftVec) := _GetProcAddress('sws_shiftVec');
    Pointer(sws_cloneVe) := _GetProcAddress('sws_cloneVec');
    Pointer(sws_printVec2) := _GetProcAddress('sws_printVec2');
    Pointer(sws_freeVec) := _GetProcAddress('sws_freeVec');
    Pointer(sws_getDefaultFilter) := _GetProcAddress('sws_getDefaultFilter');
    Pointer(sws_freeFilter) := _GetProcAddress('sws_freeFilter');
    Pointer(sws_getCachedContext) := _GetProcAddress('sws_getCachedContext');
    Pointer(sws_convertPalette8ToPacked32) := _GetProcAddress('sws_convertPalette8ToPacked32');
    Pointer(sws_convertPalette8ToPacked24) := _GetProcAddress('sws_convertPalette8ToPacked24');
    Pointer(sws_get_class) := _GetProcAddress('sws_get_class');   
    Pointer(sws_scale_frame) := _GetProcAddress('sws_scale_frame');
    Pointer(sws_frame_start) := _GetProcAddress('sws_frame_start');
    Pointer(sws_frame_end) := _GetProcAddress('sws_frame_end');
    Pointer(sws_send_slice) := _GetProcAddress('sws_send_slice');
    Pointer(sws_receive_slice) := _GetProcAddress('sws_receive_slice');
    Pointer(sws_receive_slice_alignment) := _GetProcAddress('sws_receive_slice_alignment');

    //.........................................

    Result := True;

end;

end.
